
/* this file is a part of sori software, (C) Hwang chi-duck 1997               */

#include <gtk/gtklabel.h>
#include <gtk/gtkmain.h>
#include <gtk/gtksignal.h>
#include "button.h"
 
enum {
  RIGHT_CLICK,
  LAST_SIGNAL
};

static void gtk_button_size_request (GtkWidget      *widget,
			 GtkRequisition *requisition);
static void gtk_button_size_allocate (GtkWidget     *widget,
			  GtkAllocation *allocation);
static gint gtk_button_expose (GtkWidget      *widget,
		   GdkEventExpose *event);
static void gtk_button_realize (GtkWidget *widget);
static void gtk_button_destroy(GtkObject *object);
static void gtk_button_class_init (GtkSoriButtonClass *klass);
static void gtk_button_init (GtkSoriButton *button);
static void gtk_button_draw (GtkWidget    *widget, GdkRectangle *area);
static void gtk_real_button_pressed   (GtkButton        *button);
static void gtk_real_button_released  (GtkButton        *button);
static void gtk_real_button_enter     (GtkButton        *button);
static void gtk_real_button_leave     (GtkButton        *button);
static void sori_draw_pixmap(GtkWidget *w, int x, int y, GdkPixmap *pixmap, GdkBitmap *mask);
static void gtk_sori_button_clear_and_draw(GtkWidget *button);
static gint gtk_sori_button_button_release (GtkWidget      *widget, GdkEventButton *event);
static gint gtk_sori_button_button_press (GtkWidget      *widget, GdkEventButton *event);


static GtkButtonClass *parent_class;
static gint button_signals[LAST_SIGNAL] = { 0 };


guint
gtk_sori_button_get_type ()
{
  static guint button_type = 0;

  if (!button_type)
    {
      GtkTypeInfo button_info =
      {
	"GtkSoriButton",
	sizeof (GtkSoriButton),
	sizeof (GtkSoriButtonClass),
	(GtkClassInitFunc) gtk_button_class_init,
	(GtkObjectInitFunc) gtk_button_init,
	(GtkArgFunc) NULL,
      };

      button_type = gtk_type_unique (gtk_button_get_type (), &button_info);
    }

  return button_type;
}

static void
gtk_button_class_init (GtkSoriButtonClass *klass)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  GtkContainerClass *container_class;
  GtkButtonClass *button_class;

  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;
  container_class = (GtkContainerClass*) klass;
  button_class = (GtkButtonClass*) klass;

  parent_class = gtk_type_class (gtk_button_get_type ());


  button_signals[0] = gtk_signal_new ("right_clicked",
				      GTK_RUN_FIRST,
				      object_class->type,
				      GTK_SIGNAL_OFFSET (GtkSoriButtonClass, right_clicked),
				      gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
  //button_signals[4] = gtk_signal_new ("leave",
//				      GTK_RUN_FIRST,
//				      object_class->type,
//				      GTK_SIGNAL_OFFSET (GtkButtonClass, leave),
//				      gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);

  gtk_object_class_add_signals (object_class, button_signals, LAST_SIGNAL);

  object_class->destroy = gtk_button_destroy;

  widget_class->draw = gtk_button_draw;
  widget_class->draw_focus = NULL;
  widget_class->draw_default = NULL;
  widget_class->size_request = gtk_button_size_request;
  widget_class->size_allocate = gtk_button_size_allocate;
  widget_class->expose_event = gtk_button_expose;
  widget_class->realize = gtk_button_realize;
  widget_class->button_press_event = gtk_sori_button_button_press;
  widget_class->button_release_event = gtk_sori_button_button_release;
  //widget_class->enter_notify_event = gtk_button_enter_notify;
  //widget_class->leave_notify_event = gtk_button_leave_notify;
  //widget_class->focus_in_event = gtk_button_focus_in;
  //widget_class->focus_out_event = gtk_button_focus_out;

  button_class->pressed = gtk_real_button_pressed;
  button_class->released = gtk_real_button_released;
  //klass->clicked = NULL;
  button_class->enter = gtk_real_button_enter;
  button_class->leave = gtk_real_button_leave;
}

static void
gtk_button_init (GtkSoriButton *button)
{
  GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
  button->str = NULL;
  gtk_container_border_width(GTK_CONTAINER(button), 0);

}

GtkWidget*
gtk_sori_button_new ()
{
  return GTK_WIDGET (gtk_type_new (gtk_sori_button_get_type ()));
}

GtkWidget*
gtk_sori_button_new_with_label (gchar *label, GdkGC *gc, GdkFont *font, int width, int height)
{
  GtkWidget *button;
  GtkSoriButton *soributton;

  button = gtk_sori_button_new ();
  soributton = GTK_SORI_BUTTON(button);
  soributton->str = g_strdup(label);
  soributton->type = SoriButtonTypeLabel;
  soributton->width = width;
  soributton->height = height;
  soributton->gc = gc;
  soributton->font = font;

  return button;
}

GtkWidget *
gtk_sori_button_new_with_pixmap(GdkPixmap *pixmap1, GdkBitmap *mask1,
				               GdkPixmap *pixmap2, GdkBitmap *mask2, int width, int height)
{
	GtkWidget *button;
	GtkSoriButton *soributton;
  button = gtk_sori_button_new ();
  soributton = GTK_SORI_BUTTON(button);
  soributton->pixmap1 = pixmap1;
  soributton->pixmap2 = pixmap2;
  soributton->mask1 = mask1;
  soributton->mask2 = mask2;
  soributton->width = width;
  soributton->height = height;
  //g_print("%p %p\n", pixmap1, pixmap2);
  soributton->type = SoriButtonTypePixmap;

  return button;
}



static void
gtk_button_size_allocate (GtkWidget     *widget,
			  GtkAllocation *allocation)
{
  //GtkButton *button;
  //GtkAllocation child_allocation;
  //g_print("%s\n", __FUNCTION__);

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_SORI_BUTTON (widget));
  g_return_if_fail (allocation != NULL);

  widget->allocation = *allocation;
  widget->allocation.width = widget->requisition.width;
  widget->allocation.height = widget->requisition.height;
  //g_print("%d %d %d %d\n", widget->allocation.x, widget->allocation.y,
  //	widget->allocation.width, widget->allocation.height);

  if (GTK_WIDGET_REALIZED (widget))
    gdk_window_move_resize (widget->window,
			    widget->allocation.x,
			    widget->allocation.y,
			    widget->allocation.width,
			    widget->allocation.height);
  gtk_button_draw((widget), NULL);

}

static void
gtk_button_size_request (GtkWidget      *widget,
			 GtkRequisition *requisition)
{
  GtkSoriButton *soributton;
  int width, height;

  //g_print("%s\n", __FUNCTION__);
  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_SORI_BUTTON (widget));
  g_return_if_fail (requisition != NULL);

  soributton = GTK_SORI_BUTTON (widget);
  if(soributton->width > 0 && soributton->height > 0) {
	requisition->width = soributton->width;
	requisition->height = soributton->height;
	return;
  }
  if(soributton->type == SoriButtonTypePixmap) {
      gdk_window_get_size (soributton->pixmap1, &width, &height);
      requisition->width = width + 2;
      requisition->height = height + 2;
  } else {
	 width = gdk_string_width(soributton->font, soributton->str);
	 height = soributton->font->ascent + soributton->font->descent;
      requisition->width = width + 2;
      requisition->height = height + 2;
    }
}

static void
gtk_button_draw (GtkWidget    *widget,
		 GdkRectangle *area)
{
  GtkSoriButton *soributton;
  //g_print("%s\n", __FUNCTION__);

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_SORI_BUTTON (widget));

  if (GTK_WIDGET_DRAWABLE (widget))
    {
      soributton = GTK_SORI_BUTTON (widget);
	  if(soributton->type == SoriButtonTypePixmap) {
		switch(GTK_WIDGET_STATE(widget)) {
			case GTK_STATE_PRELIGHT:
				sori_draw_pixmap(widget, 0, 0, soributton->pixmap2, soributton->mask2);
				break;
			case GTK_STATE_ACTIVE:
				sori_draw_pixmap(widget, 2, 2, soributton->pixmap2, soributton->mask2);
				break;
			default:
				sori_draw_pixmap(widget, 0, 0, soributton->pixmap1, soributton->mask1);
				break;
		}
	  } else {
		switch(GTK_WIDGET_STATE(widget)) {
			case GTK_STATE_ACTIVE:
				gdk_draw_string(widget->window, soributton->font, soributton->gc, 2, 2 + soributton->font->ascent, soributton->str); 
				break;
			default:
				gdk_draw_string(widget->window, soributton->font, soributton->gc, 0, soributton->font->ascent, soributton->str); 
				break;
		}
	  }
    }
}

static void
gtk_sori_button_clear_and_draw(GtkWidget *button)
{
	gdk_window_clear(button->window);
	gtk_button_draw(button, NULL);
}

void
sori_draw_pixmap(GtkWidget *w, int x, int y, GdkPixmap *pixmap, GdkBitmap *mask)
{
  //g_print("%s\n", __FUNCTION__);
	if(mask) {
	  gdk_gc_set_clip_mask (w->style->black_gc, mask);
	  if(x != 0 || y != 0) gdk_gc_set_clip_origin(w->style->black_gc, x, y);
	}
	gdk_draw_pixmap (w->window,
		       w->style->black_gc,
		       pixmap,
		       0, 0, x, y, -1, -1);
      if (mask)
	{
	  gdk_gc_set_clip_mask (w->style->black_gc, NULL);
	  if(x != 0 || y != 0) gdk_gc_set_clip_origin(w->style->black_gc, 0, 0);
	}
}

static void
gtk_real_button_pressed (GtkButton *button)
{
  GtkStateType new_state;
  //g_print("%s\n", __FUNCTION__);

  g_return_if_fail (button != NULL);
  g_return_if_fail (GTK_IS_BUTTON (button));

  button->button_down = TRUE;

  new_state = (button->in_button ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL);

  if (GTK_WIDGET_STATE (button) != new_state)
    {
      gtk_widget_set_state (GTK_WIDGET (button), new_state);
	  gtk_sori_button_clear_and_draw(GTK_WIDGET(button));
    }
}

static void
gtk_real_button_released (GtkButton *button)
{
  GtkStateType new_state;
  //g_print("%s\n", __FUNCTION__);

  g_return_if_fail (button != NULL);
  g_return_if_fail (GTK_IS_BUTTON (button));

  if (button->button_down)
    {
      button->button_down = FALSE;

      if (button->in_button)
	gtk_button_clicked (button);

      new_state = (button->in_button ? GTK_STATE_PRELIGHT : GTK_STATE_NORMAL);

      if (GTK_WIDGET_STATE (button) != new_state)
	{
	  gtk_widget_set_state (GTK_WIDGET (button), new_state);
	  gtk_sori_button_clear_and_draw (GTK_WIDGET (button));
	}
    }
}
static void
gtk_real_button_enter (GtkButton *button)
{
  GtkStateType new_state;
  //g_print("%s\n", __FUNCTION__);

  g_return_if_fail (button != NULL);
  g_return_if_fail (GTK_IS_BUTTON (button));

  new_state = (button->button_down ? GTK_STATE_ACTIVE : GTK_STATE_PRELIGHT);

  if (GTK_WIDGET_STATE (button) != new_state)
    {
      gtk_widget_set_state (GTK_WIDGET (button), new_state);
	  gtk_sori_button_clear_and_draw (GTK_WIDGET (button));
    }
}

static void
gtk_real_button_leave (GtkButton *button)
{
  g_return_if_fail (button != NULL);
  g_return_if_fail (GTK_IS_BUTTON (button));
  //g_print("%s\n", __FUNCTION__);

  if (GTK_WIDGET_STATE (button) != GTK_STATE_NORMAL)
    {
      gtk_widget_set_state (GTK_WIDGET (button), GTK_STATE_NORMAL);
	  gtk_sori_button_clear_and_draw (GTK_WIDGET (button));
    }
}

static gint
gtk_button_expose (GtkWidget      *widget,
		   GdkEventExpose *event)
{

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_BUTTON (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);
  //g_print("%s\n", __FUNCTION__);

  if (GTK_WIDGET_DRAWABLE (widget))
    {
      gtk_button_draw (widget, NULL);
    }

  return FALSE;
}
static void
gtk_button_realize (GtkWidget *widget)
{
  GtkButton *button;
  GdkWindowAttr attributes;
  gint attributes_mask;
  //g_print("%s\n", __FUNCTION__);

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_BUTTON (widget));

  button = GTK_BUTTON (widget);
  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.x = widget->allocation.x;
  attributes.y = widget->allocation.y;
  attributes.width = widget->allocation.width;
  attributes.height = widget->allocation.height;
  attributes.wclass = GDK_INPUT_OUTPUT;
  attributes.visual = gtk_widget_get_visual (widget);
  attributes.colormap = gtk_widget_get_colormap (widget);
  attributes.event_mask = gtk_widget_get_events (widget);
  attributes.event_mask |= (GDK_EXPOSURE_MASK |
			    GDK_BUTTON_PRESS_MASK |
			    GDK_BUTTON_RELEASE_MASK |
			    GDK_ENTER_NOTIFY_MASK |
			    GDK_LEAVE_NOTIFY_MASK);

  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

  widget->window = gdk_window_new (widget->parent->window, &attributes, attributes_mask);
  gdk_window_set_user_data (widget->window, button);

  widget->style = gtk_style_attach (widget->style, widget->window);
  gdk_window_set_back_pixmap (widget->window, NULL, TRUE);
}

static gint
gtk_sori_button_button_press (GtkWidget      *widget,
			 GdkEventButton *event)
{
  GtkSoriButton *button;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_SORI_BUTTON (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (event->type == GDK_BUTTON_PRESS)
    {
      button = GTK_SORI_BUTTON (widget);

      if (event->button == 1)
	{
	  gtk_grab_add (GTK_WIDGET (button));
	  gtk_button_pressed (GTK_BUTTON(button));
	} else if(event->button == 3) {
		gtk_grab_add(GTK_WIDGET(button));
	  //gtk_button_pressed (GTK_BUTTON(button));
	}

    }

  return TRUE;
}

static void
gtk_button_destroy(GtkObject *object)
{
	GtkSoriButton *button;

	button = (GtkSoriButton*)object;

	if(button->str) {
		g_free(button->str);
	} else {
		gdk_pixmap_destroy(button->pixmap1);
		gdk_pixmap_destroy(button->mask1);
		gdk_pixmap_destroy(button->pixmap2);
		gdk_pixmap_destroy(button->mask2);
	}

	gdk_gc_destroy(button->gc);
	gdk_font_free(button->font);

  if (GTK_WIDGET (object)->parent &&
      GTK_WIDGET_MAPPED (object))
    gtk_widget_unmap (GTK_WIDGET (object));


  if (GTK_OBJECT_CLASS (parent_class)->destroy)
    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static gint
gtk_sori_button_button_release (GtkWidget      *widget,
			   GdkEventButton *event)
{
  GtkSoriButton *button;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_SORI_BUTTON (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (event->button == 1)
    {
      button = GTK_SORI_BUTTON (widget);
      gtk_grab_remove (GTK_WIDGET (button));
      gtk_button_released (GTK_BUTTON(button));
    } else if(event->button == 3) {
		gtk_grab_remove(widget);
		if(GTK_BUTTON(widget)->in_button) gtk_signal_emit(GTK_OBJECT(widget), button_signals[RIGHT_CLICK]);
	}

  return TRUE;
}
