
/* this file is a part of sori software, (C) Hwang chi-duck 1997               */

#include "sori.h"
#include "dialog.h"

#include "gauge.h"
#include "timidity_c.h"
#include "mp3_c.h"
#include "mod_c.h"


GtkWidget *fs_dialog = NULL;
GtkWidget *option_dialog = NULL;


void
minimize_cb(GtkWidget *w)
{
	GtkWidget *top = gtk_widget_get_toplevel(w);
	gtk_widget_hide (top);
}

void
close_cb(GtkWidget *w)
{
	gtk_main_quit();
}

void
next_song_cb(GtkWidget *w)
{
	play_next();
}

void
previous_song_cb(GtkWidget *w)
{
	play_previous();
}

void
fast_forward_cb(GtkWidget *w)
{
	switch(song_type) {
		case MIDI:
			timidity_forward();
			break;
		case MP3:
			mp3_forward();
			break;
		case MOD:
			mod_forward();
			break;
		default:
			break;
	}
}

void
rewind_cb(GtkWidget *w)
{
	play_song(g_slist_nth(filenames, current_file)->data);
}

void
song_position_change_cb(GtkWidget *w, int position)
{
	float percent;
	//g_print("%s: %d\n", __FUNCTION__, position);
	percent = 100.0* (float)position / ((GtkGauge*)w)->width;
	change_position(percent);
}

void
play_list_position_cb(GtkWidget *w, int position)
{
	if(nfiles == 0) return;
	current_file = ((float)nfiles) * position / ((GtkGauge*)w)->width;
	//g_print("%s: %d %d\n", __FUNCTION__, position, current_file);
	select_update();
}

void
help_cb(GtkWidget *w)
{
	GtkWidget *message, *button;
	GtkDialog *about_window = (GtkDialog*)gtk_dialog_new();
	message = gtk_label_new("Sorry, not yet.\n\
	Please read the documents from yamp\n\
	if you want to know the format of .design files.\n");
	gtk_container_add(GTK_CONTAINER(about_window->vbox), message);
	gtk_widget_show(message);
	button = gtk_button_new_with_label("Close");
	gtk_box_pack_end(GTK_BOX(about_window->action_area), button, FALSE, FALSE, 10);
	gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(about_window));
	gtk_widget_show(button);
	gtk_widget_show((GtkWidget *)about_window);
}

void
about_cb(GtkWidget *w)
{
	GtkWidget *message, *button;
	GtkDialog *about_window = (GtkDialog*)gtk_dialog_new();
	message = gtk_label_new("This is sori 0.1 - \ngtk+ front end for sajber, amp, timidity and mikmod engine.\n\
	(C) Chiduck Hwang 1997\n");
	gtk_container_add(GTK_CONTAINER(about_window->vbox), message);
	gtk_widget_show(message);
	button = gtk_button_new_with_label("Close");
	gtk_box_pack_end(GTK_BOX(about_window->action_area), button, FALSE, FALSE, 10);
	gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(about_window));
	gtk_widget_show(button);
	gtk_widget_show((GtkWidget *)about_window);
}

void
options_cb(GtkWidget *w)
{
	if(!option_dialog) {
		option_dialog = option_dialog_create();
	} else if(GTK_WIDGET_VISIBLE(option_dialog)) {
		gdk_window_raise(option_dialog->window);
		return;
	}
	gtk_widget_show(option_dialog);
}

void
disk_operations_cb(GtkWidget *w)
{
	if(!fs_dialog) {
		fs_dialog = fs_dialog_create();
	} else if(GTK_WIDGET_VISIBLE(fs_dialog)) {
		gdk_window_raise(fs_dialog->window);
		return;
	}
	gtk_widget_show(fs_dialog);
}

void
song_information_cb(GtkWidget *w)
{
}

void
pause_cb(GtkWidget *w)
{
	if(not_play) return;
	paused = !paused;
	switch(song_type) {
		case MIDI:
			timidity_pause();
			break;
		case MP3:
			mp3_pause();
			break;
		case MOD:
			mod_pause();
			break;
		default:
			break;
	}
}


void
stop_cb(GtkWidget *w)
{
	if(not_play) return;
	kill_player();
}


void
play_cb(GtkWidget *w)
{
	if(nfiles == 0) return;
	if(not_play) {
		if(current_file < 0) current_file = 0;
		if(current_file >= nfiles) current_file = nfiles - 1;
		play_song(g_slist_nth(filenames, current_file)->data);
	}
	if(paused) {
		pause_cb(w);
	}
}

void
window_destroyed(GtkWidget *w, GtkWidget **window)
{
	*window = NULL;
}

void
close_dialog(GtkWidget *w)
{
	GtkWidget *top;

	top = gtk_widget_get_toplevel(w);
	gtk_widget_hide(top);
}

void
toggle_parameter(GtkWidget *w, int *parameter)
{
	*parameter = !*parameter;
}
