
/* this file is a part of sori software, (C) Hwang chi-duck 1997               */

#include "gauge.h"
#include <gtk/gtksignal.h>

#define BORDER 0

enum {
	PRESSED,
	RIGHT_PRESSED,
	LAST_SIGNAL
};

typedef void (*GtkGaugeSignal1) (GtkObject *object,
				 gint       arg1,
				 gpointer   data);

static void gtk_gauge_marshal_signal_1     (GtkObject         *object,
					    GtkSignalFunc      func,
					    gpointer           func_data,
					    GtkArg            *args);

static GdkColor *red_color;
static GdkColor *black_color;

static GtkWidgetClass *parent_class;
static gint gauge_signals[LAST_SIGNAL] = { 0 };

static void gtk_gauge_destroy(GtkObject *object);
static void gtk_gauge_size_request (GtkWidget      *widget,
			 GtkRequisition *requisition);
static void gtk_gauge_size_allocate (GtkWidget     *widget,
			  GtkAllocation *allocation);
static gint gtk_gauge_expose (GtkWidget      *widget,
		   GdkEventExpose *event);
static void gtk_gauge_realize (GtkWidget *widget);
static void gtk_gauge_draw (GtkWidget    *widget, GdkRectangle *area);

static void gtk_gauge_class_init (GtkGaugeClass *klass);
static void gtk_gauge_init (GtkGauge *gauge);
static gint gtk_gauge_button_press (GtkWidget      *widget, GdkEventButton *event);

guint
gtk_gauge_get_type ()
{
  static guint gauge_type = 0;

  if (!gauge_type)
    {
      GtkTypeInfo gauge_info =
      {
	"GtkGauge",
	sizeof (GtkGauge),
	sizeof (GtkGaugeClass),
	(GtkClassInitFunc) gtk_gauge_class_init,
	(GtkObjectInitFunc) gtk_gauge_init,
	(GtkArgFunc) NULL,
      };

      gauge_type = gtk_type_unique (gtk_widget_get_type (), &gauge_info);
    }

  return gauge_type;
}

static void
gtk_gauge_class_init (GtkGaugeClass *klass)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;

  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;

  parent_class = gtk_type_class (gtk_widget_get_type ());


  gauge_signals[0] = gtk_signal_new ("pressed",
				     GTK_RUN_LAST,
				     object_class->type,
				     GTK_SIGNAL_OFFSET (GtkGaugeClass, pressed),
				     gtk_gauge_marshal_signal_1, GTK_TYPE_NONE, 1,
				     GTK_TYPE_INT);
  gauge_signals[1] = gtk_signal_new ("right_pressed",
				      GTK_RUN_FIRST,
				      object_class->type,
				      GTK_SIGNAL_OFFSET (GtkGaugeClass, right_pressed),
				      gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
  //button_signals[4] = gtk_signal_new ("leave",
//				      GTK_RUN_FIRST,
//				      object_class->type,
//				      GTK_SIGNAL_OFFSET (GtkButtonClass, leave),
//				      gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);

  gtk_object_class_add_signals (object_class, gauge_signals, LAST_SIGNAL);

  object_class->destroy = gtk_gauge_destroy;

  widget_class->draw = gtk_gauge_draw;
  widget_class->size_request = gtk_gauge_size_request;
  widget_class->size_allocate = gtk_gauge_size_allocate;
  widget_class->expose_event = gtk_gauge_expose;
  widget_class->realize = gtk_gauge_realize;
  widget_class->button_press_event = gtk_gauge_button_press;
  //widget_class->button_release_event = gtk_gauge_button_release;
  //widget_class->enter_notify_event = gtk_button_enter_notify;
  //widget_class->leave_notify_event = gtk_button_leave_notify;
  //widget_class->focus_in_event = gtk_button_focus_in;
  //widget_class->focus_out_event = gtk_button_focus_out;

  red_color = g_new(GdkColor, 1);
  red_color->red = 65535;
  red_color->blue = red_color->green = 65535/4;
  gdk_color_alloc(gdk_colormap_get_system(), red_color);

  black_color = g_new(GdkColor, 1);
  black_color->red = 0;
  black_color->blue = black_color->green = 0;
  gdk_color_alloc(gdk_colormap_get_system(), black_color);

}

static void
gtk_gauge_init (GtkGauge *gauge)
{
  GTK_WIDGET_UNSET_FLAGS (gauge, GTK_CAN_FOCUS);
}

GtkWidget *
gtk_gauge_new(int width, int height, GtkAdjustment *adj)
{
	GtkWidget *widget;
	GtkGauge *gauge;

	widget = gtk_type_new (gtk_gauge_get_type ());
	gauge = GTK_GAUGE(widget);
	gauge->position  = 0;
	gauge->width = width;
	gauge->height = height;

	return widget;
}

void
gtk_gauge_set_value(GtkGauge *gauge, float val)
{
	int value, old_value;
	if(val < 0) val = 0;
	else if(val > 1) val = 1;
	value = val * gauge->width;
	old_value = gauge->position;
	if(value == old_value) return;
	if(value > old_value) {
		gdk_gc_set_foreground(gauge->gc, red_color);
		gdk_draw_rectangle(((GtkWidget*)gauge)->window, gauge->gc, TRUE, old_value, BORDER, value - old_value, gauge->height);
	} else {
		gdk_gc_set_foreground(gauge->gc, black_color);
		gdk_draw_rectangle(((GtkWidget *)gauge)->window, gauge->gc, TRUE, value , BORDER, old_value - value, gauge->height);
	}
	gauge->position = value;
}


static void 
gtk_gauge_size_request (GtkWidget      *widget, GtkRequisition *requisition)
{
	GtkGauge *gauge;
	//g_print("==%s\n", __FUNCTION__);

	gauge = GTK_GAUGE(widget);
	requisition->width = gauge->width + 2*BORDER;
	requisition->height = gauge->height + 2*BORDER;
	//g_print("%d %d\n", gauge->width, gauge->height);
}

static void 
gtk_gauge_size_allocate (GtkWidget     *widget, GtkAllocation *allocation)
{
	//g_print("==%s\n", __FUNCTION__);
	widget->allocation = *allocation;
  if (GTK_WIDGET_REALIZED (widget))
    gdk_window_move_resize (widget->window,
			    widget->allocation.x,
			    widget->allocation.y,
			    widget->allocation.width,
			    widget->allocation.height);
  gtk_gauge_draw((widget), NULL);
}

static gint 
gtk_gauge_expose (GtkWidget      *widget, GdkEventExpose *event)
{
	gtk_gauge_draw(widget, NULL);
	return TRUE;
}

static void 
gtk_gauge_realize (GtkWidget *widget)
{
	GtkGauge *gauge;
  GdkWindowAttr attributes;
  gint attributes_mask;
  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_GAUGE (widget));
  //g_print("======%s\n", __FUNCTION__);

  gauge = GTK_GAUGE (widget);
  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.x = widget->allocation.x;
  attributes.y = widget->allocation.y;
  attributes.width = widget->allocation.width;
  attributes.height = widget->allocation.height;
  attributes.wclass = GDK_INPUT_OUTPUT;
  attributes.visual = gtk_widget_get_visual (widget);
  attributes.colormap = gtk_widget_get_colormap (widget);
  attributes.event_mask = gtk_widget_get_events (widget);
  attributes.event_mask |= (GDK_EXPOSURE_MASK |
			    GDK_BUTTON_PRESS_MASK |
			    //GDK_BUTTON_RELEASE_MASK |
			    GDK_ENTER_NOTIFY_MASK |
			    GDK_LEAVE_NOTIFY_MASK);

  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

  widget->window = gdk_window_new (widget->parent->window, &attributes, attributes_mask);
  gdk_window_set_user_data (widget->window, gauge);

  widget->style = gtk_style_attach (widget->style, widget->window);
  gdk_window_set_back_pixmap (widget->window, NULL, TRUE);
	gauge->gc = gdk_gc_new(widget->window);
}

static void 
gtk_gauge_draw (GtkWidget    *widget, GdkRectangle *area)
{
	GtkGauge *gauge;
	int x, width;
	//g_print("===============================%s\n", __FUNCTION__);
	gauge = GTK_GAUGE(widget);

	x = BORDER; 
	width = gauge->position;

	gdk_gc_set_foreground(gauge->gc, red_color);
	gdk_draw_rectangle(widget->window, gauge->gc, TRUE, x, BORDER, width, gauge->height);
	gdk_gc_set_foreground(gauge->gc, black_color);
	gdk_draw_rectangle(widget->window, gauge->gc, TRUE, x + width, BORDER, gauge->width - width, gauge->height);
}

static gint 
gtk_gauge_button_press (GtkWidget      *widget, GdkEventButton *event)
{
	GtkGauge *gauge;

	gauge = GTK_GAUGE(widget);
	if(event->button == 1) {
		//gauge->adjustment->value = ((double)event->x - BORDER)/gauge->width;
		//if(gauge->adjustment->value < 0) gauge->adjustment->value = 0;
		gtk_signal_emit(GTK_OBJECT(widget), gauge_signals[PRESSED], (int)event->x);
	} else if(event->button == 3) {
		gtk_signal_emit(GTK_OBJECT(widget), gauge_signals[RIGHT_PRESSED]);
	}
	return TRUE;
}

static void
gtk_gauge_destroy(GtkObject *object)
{
	GtkGauge *gauge;

	gauge = (GtkGauge*)object;
	gdk_gc_destroy(gauge->gc);
  if (GTK_WIDGET (object)->parent &&
      GTK_WIDGET_MAPPED (object))
    gtk_widget_unmap (GTK_WIDGET (object));


  if (GTK_OBJECT_CLASS (parent_class)->destroy)
    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static void
gtk_gauge_marshal_signal_1 (GtkObject      *object,
			    GtkSignalFunc   func,
			    gpointer        func_data,
			    GtkArg         *args)
{
  GtkGaugeSignal1 rfunc;

  rfunc = (GtkGaugeSignal1) func;

  (* rfunc) (object, GTK_VALUE_INT(args[0]), func_data);
}

