
/* this file is a part of sori software, (C) Hwang chi-duck 1997              */
/* This file is derived from gif plugin for gimp                              */
/* The following is copyright notices from that file                          */

/* GIF loading and saving file filter for the Gimp version 0.99+
 *  -Peter Mattis & Spencer Kimball
 *
 * version 1.99.3 - 97/05/18
 *
 * Additional transparent/animated GIF stuff is by Adam D. Moss,
 * adam@foxbox.org - still in progress, 97/05/18
 *
 *
 * This filter uses code taken from the "giftopnm" and "ppmtogif" programs
 *  which are part of the "netpbm" package.
 */


/*
 *  "The Graphics Interchange Format(c) is the Copyright property of
 *  CompuServe Incorporated.  GIF(sm) is a Service Mark property of
 *  CompuServe Incorporated." 
 *
 */



/* Copyright notice for code which this plugin was derived from - */
/* +-------------------------------------------------------------------+ */
/* | Copyright 1990, 1991, 1993, David Koblas.  (koblas@netcom.com)    | */
/* |   Permission to use, copy, modify, and distribute this software   | */
/* |   and its documentation for any purpose and without fee is hereby | */
/* |   granted, provided that the above copyright notice appear in all | */
/* |   copies and that both that copyright notice and this permission  | */
/* |   notice appear in supporting documentation.  This software is    | */
/* |   provided "as is" without express or implied warranty.           | */
/* +-------------------------------------------------------------------+ */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>

#include <gdk/gdk.h>
#include <gdk/gdkprivate.h>
#include "anipixmap.h"


#define MAXCOLORMAPSIZE  256


#define CM_RED           0
#define CM_GREEN         1
#define CM_BLUE          2

#define MAX_LWZ_BITS     12

#define INTERLACE          0x40
#define LOCALCOLORMAP      0x80
#define BitSet(byte, bit)  (((byte) & (bit)) == (bit))

#define ReadOK(file,buffer,len) (fread(buffer, len, 1, file) != 0)
#define LM_to_uint(a,b)         (((b)<<8)|(a))

#define GRAYSCALE        1
#define COLOR            2

typedef unsigned char CMap[3][MAXCOLORMAPSIZE];

static struct
  {
    unsigned int Width;
    unsigned int Height;
    CMap ColorMap;
    unsigned int BitPixel;
    unsigned int ColorResolution;
    unsigned int Background;
    unsigned int AspectRatio;
    /*
     **
     */
    int IsGrayScale;
  }
GifScreen;

static struct
  {
    int transparent;
    int delayTime;
    int inputFlag;
    int disposal;
  }
Gif89 =
{

  -1, -1, -1, 0
};

static int verbose = FALSE;

static int ReadColorMap (FILE *, int, CMap, int *);
static int DoExtension (FILE *, int);
static int GetDataBlock (FILE *, unsigned char *);
static int GetCode (FILE *, int, int);
static int LWZReadByte (FILE *, int, int);
static GdkPixmap *ReadImage (FILE *, GdkWindow *, GdkBitmap ** mask,
			     GdkColor * bg, int, int, CMap, int, int, int);

GdkPixmap *
gdk_pixmap_create_from_gif (GdkWindow * window,
			    GdkBitmap ** mask,
			    GdkColor * transparent_color,
			    gchar * filename)
{
  FILE *fd;
  unsigned char buf[16];
  unsigned char c;
  CMap localColorMap;
  int grayScale;
  int useGlobalColormap;
  int bitPixel;
  int imageCount = 0;
  char version[4];

  Gif89.transparent = -1;
  Gif89.delayTime = -1;
  Gif89.inputFlag = -1;
  Gif89.disposal = 0;

  if (!window)
    window = (GdkWindow *) & gdk_root_parent;


  fd = fopen (filename, "rb");
  if (!fd)
    {
      g_warning ("GIF: can't open \"%s\"\n", filename);
      return NULL;
    }
  if (!ReadOK (fd, buf, 6))
    {
      g_warning ("GIF: error reading magic number\n");
      return NULL;
    }
  if (strncmp ((char *) buf, "GIF", 3) != 0)
    {
      g_warning ("GIF: not a GIF file\n");
      return NULL;
    }
  strncpy (version, (char *) buf + 3, 3);
  version[3] = '\0';

  if ((strcmp (version, "87a") != 0) && (strcmp (version, "89a") != 0))
    {
      g_warning ("GIF: bad version number, not '87a' or '89a'\n");
      return NULL;
    }
  if (!ReadOK (fd, buf, 7))
    {
      g_warning ("GIF: failed to read screen descriptor\n");
      return NULL;
    }
  GifScreen.Width = LM_to_uint (buf[0], buf[1]);
  GifScreen.Height = LM_to_uint (buf[2], buf[3]);
  GifScreen.BitPixel = 2 << (buf[4] & 0x07);
  GifScreen.ColorResolution = (((buf[4] & 0x70) >> 3) + 1);
  GifScreen.Background = buf[5];
  GifScreen.AspectRatio = buf[6];

  if (BitSet (buf[4], LOCALCOLORMAP))
    {
      /* Global Colormap */
      if (ReadColorMap (fd, GifScreen.BitPixel, GifScreen.ColorMap, &GifScreen.IsGrayScale))
	{
	  g_warning ("GIF: error reading global colormap\n");
	  return NULL;
	}
    }
  if (GifScreen.AspectRatio != 0 && GifScreen.AspectRatio != 49)
    {
      g_warning ("GIF: warning - non-square pixels\n");
    }
  for (;;)
    {
      if (!ReadOK (fd, &c, 1))
	{
	  g_warning ("GIF: EOF / read error on image data\n");
	  return NULL;
	}
      if (c == ';')
	{
	  /* GIF terminator */
	  //return image_ID;
	  return NULL;
	}
      if (c == '!')
	{			/* Extension */
	  if (!ReadOK (fd, &c, 1))
	    {
	      g_warning ("GIF: OF / read error on extention function code\n");
	      return NULL;
	    }
	  DoExtension (fd, c);
	  continue;
	}
      if (c != ',')
	{
	  /* Not a valid start character */
	  g_warning ("GIF: bogus character 0x%02x, ignoring\n", (int) c);
	  continue;
	}
      ++imageCount;

      if (!ReadOK (fd, buf, 9))
	{
	  g_warning ("GIF: couldn't read left/top/width/height\n");
	  return NULL;
	}
      useGlobalColormap = !BitSet (buf[8], LOCALCOLORMAP);

      bitPixel = 1 << ((buf[8] & 0x07) + 1);

      if (!useGlobalColormap)
	{
	  if (ReadColorMap (fd, bitPixel, localColorMap, &grayScale))
	    {
	      g_warning ("GIF: error reading local colormap\n");
	      return NULL;
	    }
	  return ReadImage (fd, window, mask, transparent_color, LM_to_uint (buf[4], buf[5]),
			    LM_to_uint (buf[6], buf[7]),
			    localColorMap, bitPixel,
			    grayScale,
			    BitSet (buf[8], INTERLACE));
	}
      else
	{
	  return ReadImage (fd, window, mask, transparent_color, LM_to_uint (buf[4], buf[5]),
			    LM_to_uint (buf[6], buf[7]),
			    GifScreen.ColorMap, GifScreen.BitPixel,
			    GifScreen.IsGrayScale,
			    BitSet (buf[8], INTERLACE));
	}

    }

  return NULL;
}

static int 
ReadColorMap (FILE * fd,
	      int number,
	      CMap buffer,
	      int *format)
{
  int i;
  unsigned char rgb[3];
  int flag;

  flag = TRUE;

  for (i = 0; i < number; ++i)
    {
      if (!ReadOK (fd, rgb, sizeof (rgb)))
	{
	  g_warning ("GIF: bad colormap\n");
	  return TRUE;
	}
      buffer[CM_RED][i] = rgb[0];
      buffer[CM_GREEN][i] = rgb[1];
      buffer[CM_BLUE][i] = rgb[2];

      flag &= (rgb[0] == rgb[1] && rgb[1] == rgb[2]);
    }

  *format = (flag) ? GRAYSCALE : COLOR;

  return FALSE;
}

static int 
DoExtension (FILE * fd,
	     int label)
{
  static guchar buf[256];
  char *str;

  switch (label)
    {
    case 0x01:			/* Plain Text Extension */
      str = "Plain Text Extension";
#ifdef notdef
      if (GetDataBlock (fd, (unsigned char *) buf) == 0);

      lpos = LM_to_uint (buf[0], buf[1]);
      tpos = LM_to_uint (buf[2], buf[3]);
      width = LM_to_uint (buf[4], buf[5]);
      height = LM_to_uint (buf[6], buf[7]);
      cellw = buf[8];
      cellh = buf[9];
      foreground = buf[10];
      background = buf[11];

      while (GetDataBlock (fd, (unsigned char *) buf) != 0)
	{
	  PPM_ASSIGN (image[ypos][xpos],
		      cmap[CM_RED][v],
		      cmap[CM_GREEN][v],
		      cmap[CM_BLUE][v]);
	  ++index;
	}

      return FALSE;
#else
      break;
#endif
    case 0xff:			/* Application Extension */
      str = "Application Extension";
      break;
    case 0xfe:			/* Comment Extension */
      str = "Comment Extension";
      while (GetDataBlock (fd, (unsigned char *) buf) != 0)
	{
	  if (verbose)
	    printf ("GIF: gif comment: %s\n", buf);
	}
      return FALSE;
    case 0xf9:			/* Graphic Control Extension */
      str = "Graphic Control Extension";
      (void) GetDataBlock (fd, (unsigned char *) buf);
      Gif89.disposal = (buf[0] >> 2) & 0x7;
      Gif89.inputFlag = (buf[0] >> 1) & 0x1;
      Gif89.delayTime = LM_to_uint (buf[1], buf[2]);
      if ((buf[0] & 0x1) != 0)
	Gif89.transparent = buf[3];

      while (GetDataBlock (fd, (unsigned char *) buf) != 0);
      return FALSE;
    default:
      str = buf;
      sprintf (buf, "UNKNOWN (0x%02x)", label);
      break;
    }

  if (verbose)
    g_warning ("GIF: got a '%s' extension\n", str);

  while (GetDataBlock (fd, (unsigned char *) buf) != 0);

  return FALSE;
}

int ZeroDataBlock = FALSE;

static int 
GetDataBlock (FILE * fd,
	      unsigned char *buf)
{
  unsigned char count;

  if (!ReadOK (fd, &count, 1))
    {
      g_warning ("GIF: error in getting DataBlock size\n");
      return -1;
    }
  ZeroDataBlock = count == 0;

  if ((count != 0) && (!ReadOK (fd, buf, count)))
    {
      g_warning ("GIF: error in reading DataBlock\n");
      return -1;
    }
  return count;
}

static int 
GetCode (FILE * fd,
	 int code_size,
	 int flag)
{
  static unsigned char buf[280];
  static int curbit, lastbit, done, last_byte;
  int i, j, ret;
  unsigned char count;

  if (flag)
    {
      curbit = 0;
      lastbit = 0;
      done = FALSE;
      return 0;
    }
  if ((curbit + code_size) >= lastbit)
    {
      if (done)
	{
	  if (curbit >= lastbit)
	    {
	      g_warning ("GIF: ran off the end of by bits\n");
	      return -1;
	    }
	  return -1;
	}
      buf[0] = buf[last_byte - 2];
      buf[1] = buf[last_byte - 1];

      if ((count = GetDataBlock (fd, &buf[2])) == 0)
	done = TRUE;

      last_byte = 2 + count;
      curbit = (curbit - lastbit) + 16;
      lastbit = (2 + count) * 8;
    }
  ret = 0;
  for (i = curbit, j = 0; j < code_size; ++i, ++j)
    ret |= ((buf[i / 8] & (1 << (i % 8))) != 0) << j;

  curbit += code_size;

  return ret;
}

static int 
LWZReadByte (FILE * fd,
	     int flag,
	     int input_code_size)
{
  static int fresh = FALSE;
  int code, incode;
  static int code_size, set_code_size;
  static int max_code, max_code_size;
  static int firstcode, oldcode;
  static int clear_code, end_code;
  static int table[2][(1 << MAX_LWZ_BITS)];
  static int stack[(1 << (MAX_LWZ_BITS)) * 2], *sp;
  register int i;

  if (flag)
    {
      set_code_size = input_code_size;
      code_size = set_code_size + 1;
      clear_code = 1 << set_code_size;
      end_code = clear_code + 1;
      max_code_size = 2 * clear_code;
      max_code = clear_code + 2;

      GetCode (fd, 0, TRUE);

      fresh = TRUE;

      for (i = 0; i < clear_code; ++i)
	{
	  table[0][i] = 0;
	  table[1][i] = i;
	}
      for (; i < (1 << MAX_LWZ_BITS); ++i)
	table[0][i] = table[1][0] = 0;

      sp = stack;

      return 0;
    }
  else if (fresh)
    {
      fresh = FALSE;
      do
	{
	  firstcode = oldcode =
	    GetCode (fd, code_size, FALSE);
	}
      while (firstcode == clear_code);
      return firstcode;
    }
  if (sp > stack)
    return *--sp;

  while ((code = GetCode (fd, code_size, FALSE)) >= 0)
    {
      if (code == clear_code)
	{
	  for (i = 0; i < clear_code; ++i)
	    {
	      table[0][i] = 0;
	      table[1][i] = i;
	    }
	  for (; i < (1 << MAX_LWZ_BITS); ++i)
	    table[0][i] = table[1][i] = 0;
	  code_size = set_code_size + 1;
	  max_code_size = 2 * clear_code;
	  max_code = clear_code + 2;
	  sp = stack;
	  firstcode = oldcode =
	    GetCode (fd, code_size, FALSE);
	  return firstcode;
	}
      else if (code == end_code)
	{
	  int count;
	  unsigned char buf[260];

	  if (ZeroDataBlock)
	    return -2;

	  while ((count = GetDataBlock (fd, buf)) > 0);

	  if (count != 0)
	    g_warning ("GIF: missing EOD in data stream (common occurence)");
	  return -2;
	}
      incode = code;

      if (code >= max_code)
	{
	  *sp++ = firstcode;
	  code = oldcode;
	}
      while (code >= clear_code)
	{
	  *sp++ = table[1][code];
	  if (code == table[0][code])
	    {
	      g_warning ("GIF: circular table entry BIG ERROR\n");
	      return -1;
	    }
	  code = table[0][code];
	}

      *sp++ = firstcode = table[1][code];

      if ((code = max_code) < (1 << MAX_LWZ_BITS))
	{
	  table[0][code] = oldcode;
	  table[1][code] = firstcode;
	  ++max_code;
	  if ((max_code >= max_code_size) &&
	      (max_code_size < (1 << MAX_LWZ_BITS)))
	    {
	      max_code_size *= 2;
	      ++code_size;
	    }
	}
      oldcode = incode;

      if (sp > stack)
	return *--sp;
    }
  return code;
}

static GdkPixmap *
ReadImage (FILE * fd,
	   GdkWindow * window,
	   GdkBitmap ** mask,
	   GdkColor * transparent_color,
	   int width,
	   int height,
	   CMap cmap,
	   int ncols,
	   int format,
	   int interlace)
{
  GdkPixmap *pixmap;
  GdkImage *image;
  GdkVisual *visual;
  guchar *dest, *temp;
  guchar c;
  gint xpos = 0, ypos = 0, pass = 0;
  gint v;
  gint i, j;
  gint cnt;
  GdkColor *color;
  int color_index;
  GdkGC *gc;
  GdkColormap *colormap = gdk_window_get_colormap (window);

  visual = gdk_window_get_visual (window);
  image = gdk_image_new (GDK_IMAGE_FASTEST, visual, width, height);
  gc = NULL;
  if (mask)
    {
      GdkColor tmp_color;

      *mask = gdk_pixmap_new (window, width, height, 1);
      gc = gdk_gc_new (*mask);

      tmp_color.pixel = 0;
      gdk_gc_set_foreground (gc, &tmp_color);
      gdk_draw_rectangle (*mask, gc, TRUE, 0, 0, -1, -1);

      tmp_color.pixel = ~((gulong) 0);
      gdk_gc_set_foreground (gc, &tmp_color);
    }

  /*
   **  Initialize the Compression routines
   */
  if (!ReadOK (fd, &c, 1))
    {
      g_warning ("EOF / read error on image data\n");
      return NULL;
    }
  if (LWZReadByte (fd, TRUE, c) < 0)
    {
      g_warning ("error while reading\n");
      return NULL;
    }

  dest = (guchar *) g_malloc (width * height);

  if (verbose)
    g_warning ("GIF: reading %d by %d%s GIF image, ncols=%d\n",
	       width, height, interlace ? " interlaced" : "", ncols);

  while ((v = LWZReadByte (fd, FALSE, c)) >= 0)
    {
      temp = dest + (ypos * width) + xpos;
      *temp = (guchar) v;

      ++xpos;
      if (xpos == width)
	{
	  xpos = 0;
	  if (interlace)
	    {
	      switch (pass)
		{
		case 0:
		case 1:
		  ypos += 8;
		  break;
		case 2:
		  ypos += 4;
		  break;
		case 3:
		  ypos += 2;
		  break;
		}

	      if (ypos >= height)
		{
		  ++pass;
		  switch (pass)
		    {
		    case 1:
		      ypos = 4;
		      break;
		    case 2:
		      ypos = 2;
		      break;
		    case 3:
		      ypos = 1;
		      break;
		    default:
		      goto fini;
		    }
		}
	    }
	  else
	    {
	      ++ypos;
	    }
	}
      if (ypos >= height)
	break;
    }

fini:
  if (LWZReadByte (fd, FALSE, c) >= 0)
    g_warning ("GIF: too much input data, ignoring extra...\n");

  color = g_new (GdkColor, ncols);
  for (i = 0; i < ncols; i++)
    {
      int retval;
      if (i == Gif89.transparent && transparent_color)
	{
	  color[i].red = transparent_color->red;
	  color[i].green = transparent_color->green;
	  color[i].blue = transparent_color->blue;
	}
      else
	{
	  color[i].red = cmap[0][i] << 8;
	  color[i].green = cmap[1][i] << 8;
	  color[i].blue = cmap[2][i] << 8;
	}
      retval = gdk_color_alloc (colormap, &color[i]);
      //printf("color: %d %d %d %d\n", color[i].red, color[i].green, color[i].blue, color[i].pixel);
      //printf("color: %d %d \n", i, retval);
    }

  for (j = 0; j < height; j++)
    {
      for (cnt = 0, i = 0; i < width; i++)
	{
	  color_index = dest[j * width + i];
	  //printf("%d %d: %d %d\n", i, j, color_index, color[color_index].pixel);
	  gdk_image_put_pixel (image, i, j, color[color_index].pixel);
	  if (mask && color_index == Gif89.transparent)
	    {
	      if (cnt < i)
		gdk_draw_line (*mask, gc, cnt, j, i - 1, j);
	      cnt = i + 1;
	    }
	}
      if (mask && (cnt < i))
	{
	  gdk_draw_line (*mask, gc, cnt, j, i - 1, j);
	}
    }
  if (gc)
    gdk_gc_destroy (gc);
  pixmap = gdk_pixmap_new (window, width, height, visual->depth);

  gc = gdk_gc_new (pixmap);
  if (transparent_color)
    gdk_gc_set_foreground (gc, transparent_color);
  gdk_draw_image (pixmap, gc, image, 0, 0, 0, 0, image->width, image->height);
  gdk_gc_destroy (gc);
  gdk_image_destroy (image);
  g_free (dest);
  g_free (color);

  return pixmap;
}


GtkWidget *
gtk_ani_pixmap_new_from_gif (GdkWindow * window,
			     GdkColor * transparent_color,
			     gchar * filename)
{
  FILE *fd;
  unsigned char buf[16];
  unsigned char c;
  CMap localColorMap;
  int grayScale;
  int useGlobalColormap;
  GdkBitmap *mask;
  int bitPixel;
  char version[4];
  GList *grim_list = NULL;
  AniItem *ani_item;
  GdkPixmap *pixmap;

  Gif89.transparent = -1;
  Gif89.delayTime = -1;
  Gif89.inputFlag = -1;
  Gif89.disposal = 0;

  if (!window)
    window = (GdkWindow *) & gdk_root_parent;


  fd = fopen (filename, "rb");
  if (!fd)
    {
      g_warning ("GIF: can't open \"%s\"\n", filename);
      return NULL;
    }
  if (!ReadOK (fd, buf, 6))
    {
      g_warning ("GIF: error reading magic number\n");
      return NULL;
    }
  if (strncmp ((char *) buf, "GIF", 3) != 0)
    {
      g_warning ("GIF: not a GIF file\n");
      return NULL;
    }
  strncpy (version, (char *) buf + 3, 3);
  version[3] = '\0';

  if ((strcmp (version, "87a") != 0) && (strcmp (version, "89a") != 0))
    {
      g_warning ("GIF: bad version number, not '87a' or '89a'\n");
      return NULL;
    }
  if (!ReadOK (fd, buf, 7))
    {
      g_warning ("GIF: failed to read screen descriptor\n");
      return NULL;
    }
  GifScreen.Width = LM_to_uint (buf[0], buf[1]);
  GifScreen.Height = LM_to_uint (buf[2], buf[3]);
  GifScreen.BitPixel = 2 << (buf[4] & 0x07);
  GifScreen.ColorResolution = (((buf[4] & 0x70) >> 3) + 1);
  GifScreen.Background = buf[5];
  GifScreen.AspectRatio = buf[6];

  if (BitSet (buf[4], LOCALCOLORMAP))
    {
      /* Global Colormap */
      if (ReadColorMap (fd, GifScreen.BitPixel, GifScreen.ColorMap, &GifScreen.IsGrayScale))
	{
	  g_warning ("GIF: error reading global colormap\n");
	  return NULL;
	}
    }
  if (GifScreen.AspectRatio != 0 && GifScreen.AspectRatio != 49)
    {
      g_warning ("GIF: warning - non-square pixels\n");
    }
  for (;;)
    {
      if (!ReadOK (fd, &c, 1))
	{
	  g_warning ("GIF: EOF / read error on image data\n");
	  return NULL;
	}
      if (c == ';')
	{
	  /* GIF terminator */
	  //return image_ID;
	  //return NULL;
	  break;
	}
      if (c == '!')
	{			/* Extension */
	  if (!ReadOK (fd, &c, 1))
	    {
	      g_warning ("GIF: OF / read error on extention function code\n");
	      return NULL;
	    }
	  DoExtension (fd, c);
	  continue;
	}
      if (c != ',')
	{
	  /* Not a valid start character */
	  g_warning ("GIF: bogus character 0x%02x, ignoring\n", (int) c);
	  continue;
	}

      if (!ReadOK (fd, buf, 9))
	{
	  g_warning ("GIF: couldn't read left/top/width/height\n");
	  return NULL;
	}
      useGlobalColormap = !BitSet (buf[8], LOCALCOLORMAP);

      bitPixel = 1 << ((buf[8] & 0x07) + 1);

      if (!useGlobalColormap)
	{
	  if (ReadColorMap (fd, bitPixel, localColorMap, &grayScale))
	    {
	      g_warning ("GIF: error reading local colormap\n");
	      return NULL;
	    }
	  pixmap = ReadImage (fd, window, &mask, transparent_color, LM_to_uint (buf[4], buf[5]),
			      LM_to_uint (buf[6], buf[7]),
			      localColorMap, bitPixel,
			      grayScale,
			      BitSet (buf[8], INTERLACE));
	}
      else
	{
	  pixmap = ReadImage (fd, window, &mask, transparent_color, LM_to_uint (buf[4], buf[5]),
			      LM_to_uint (buf[6], buf[7]),
			      GifScreen.ColorMap, GifScreen.BitPixel,
			      GifScreen.IsGrayScale,
			      BitSet (buf[8], INTERLACE));
	}
      ani_item = g_new (AniItem, 1);
      ani_item->pixmap = pixmap;
      ani_item->mask = mask;
      ani_item->duration = 20 * Gif89.delayTime;
      //printf("duration: %u\n", Gif89.delayTime);
      grim_list = g_list_append (grim_list, ani_item);
    }

  return ani_pixmap_new (grim_list);
}
