/*

   Name:
   MIKMODUX.C

   Description:
   Modplaying example of mikmod - bare unix? version.

   Original by MikMak <mikmak@via.nl>, 
   then Chris Conn <cconn@tohs.abacom.com>,
   then Steve McIntyre <stevem@chiark.greenend.org.uk>

   HISTORY
   =======

   v1.00 (06/12/96) - first "versioned" version

 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include "mikmod.h"

#include "ui.h"
#include "pipe.h"

#ifdef CPUTIME_SNAGGER
#include <sched.h>
#endif							/* CPUTIME_SNAGGER */

curmod cur_mod;

char *d_text[] =
{
	"            ",				/*standard, not deleted */
	" <del>      ",				/*marked for deletion */
	" <deleted>  ",				/*deleted, gone! */
	" <del error>"				/*can't delete... */
};


int quiet = 0;					/* set if quiet mode is enabled */
UBYTE md_type = 0;				/* default is a non-wavetable sound device */
void main_loop(void);
void play(char *filename);

void tickhandler(void)
{
	MP_HandleTick();			/* play 1 tick of the module */
	MD_SetBPM(mp_bpm);
}

int main(int nargc, char *nargv[])
{
	extern int quit;
	extern int play_current;
	extern float speed_constant;	/* tempo multiplier, initialised to 1 */
	/*
	   Initialize soundcard parameters.. you _have_ to do this
	   before calling MD_Init(), and it's illegal to change them
	   after you've called MD_Init()
	 */

	play_current = 1;			/* we are playing _this_ mod; jump to next 
								   when this goes to 0 */
	md_mixfreq = 44100;			/* standard mixing freq */
	md_dmabufsize = 32768;		/* standard dma buf size */
	md_mode = DMODE_16BITS | DMODE_STEREO;	/* standard mixing mode */
	md_device = 0;				/* standard device: autodetect */

	/*
	   Register the loaders we want to use..
	 */

	ML_RegisterLoader(&load_m15);	/* if you use m15load, register it as first! */
	ML_RegisterLoader(&load_mod);
	ML_RegisterLoader(&load_mtm);
	ML_RegisterLoader(&load_s3m);
	ML_RegisterLoader(&load_stm);
	ML_RegisterLoader(&load_ult);
	ML_RegisterLoader(&load_uni);
	ML_RegisterLoader(&load_669);
	ML_RegisterLoader(&load_xm);

	/*
	   Register the drivers we want to use:
	 */

	MD_RegisterDriver(&drv_nos);
	MD_RegisterDriver(&drv_raw);

	MD_RegisterDriver(&drv_vox);
	MD_RegisterPlayer(tickhandler);

	/* Parse option switches using standard getopt function: */

	if (!MD_Init()) {
		fprintf(stderr, "Driver error: %s.\n", myerr);
		return 0;
	}
	/*  initialize volume and tempo multipliers */

	speed_constant = 1.0;
	mp_volume = 200;



	/*  initialize curses interface */
	init_display();

	main_loop();

	return 1;
}

void
main_loop(void)
{

	int code;
	char filename[1024];
	fd_set readfds;
	while(1) {
		play_current = 1;
		FD_ZERO(&readfds);
		FD_SET(0, &readfds);
		if(!select(1, &readfds, NULL, NULL, NULL)) {
			break ;
		}
		ui_result = 2;
		pipe_int_read(&code);
		//fprintf(stderr,"mod:pipe_int_read %d\n",code);
		if(code == MOD_PLAY) {
			pipe_string_read(filename);
			//fprintf(stderr, "mod:playing %s\n", filename);
			play(filename);
			if(ui_result == UI_NEXT_SONG) {
				pipe_int_write(MOD_NEXT);
			}
			if(play_current) {
				pipe_int_write(MOD_END);
			} else 
				play_current = 0;
		} else if(code == MOD_STOP) {
			pipe_int_write(MOD_STOP); /* response caller */
			break;
		} else if(code == MOD_NEXT) {
			pipe_int_write(MOD_NEXT);
		} else if(code == MOD_SET_POS) {
		}
		if(quit) {
			pipe_int_write(MOD_STOP);
		}
	}
}

void
play(char *filename)
{

	UNIMOD *mf;
	/* kill audio output, as some mods sound awful left hanging... */
	MD_PatternChange();

	cur_mod.deleted = 0;
	strcpy(cur_mod.filename, filename);

	/* load the module */

	mf = ML_LoadFN(filename);

	/* didn't work -> exit with errormsg. */

	if (mf == NULL) {
		pipe_int_write(MOD_FAILURE);
		return;
	}

	/*      initialize modplayer to play this module */

	MP_Init(mf);

	update_driver_display();
	update_file_display();
	display_version();
	display_driver();
	display_file();

	cur_mod.numpat = mf->numpos;
	cur_mod.songname = mf->songname;
	cur_mod.modtype = mf->modtype;
	cur_mod.flags = mf->flags;
	update_name_display();
	pipe_int_write(MOD_NUM_PAT);
	pipe_int_write(cur_mod.numpat);
	display_name();


	/*  set the number of voices to use.. you
	   could add extra channels here (e.g. md_numchn=mf->numchn+4; )
	   to use for your own soundeffects:
	 */

	md_numchn = mf->numchn;

	/*  start playing the module: */

	MD_PlayStart();

	while (!MP_Ready() && !quit) {	/* if we have a quit signal, exit loop */

		MD_Update();

		/* no need to wait with the unix drivers */

			/* update the status display... */
		cur_mod.sngpos = mp_sngpos + 1;
		cur_mod.patpos = mp_patpos;
		display_status();
		if (!play_current) {	/*play_current=0 when the next or previous */
			break;
		}
	}
	MD_PlayStop();			/* stop playing */
	ML_Free(mf);			/* and free the module */
}
