/*

Name:
DISPLAY.C

Description:
Ncurses display section of mikmod - bare unix version.

Should deal with all the quiet options and refresh() after ncurses calls,
so just call these functions direct wherever needed. 

Steve McIntyre <stevem@chiark.greenend.org.uk>

HISTORY
=======

v1.00 (06/12/96) - first "versioned" version
v1.01 (06/12/96) - fixed update_name_display for Solaris - add "(null)" if necessary
v1.02 (16/12/96) - minor cosmetic change to display_error()

*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "mikmod.h"
#include "pipe.h"
#include "ui.h"

extern curmod cur_mod;
extern float speed_constant;   /* multiplier for tempo,initialised to 1 */

void init_display(void)
{

}

void display_version(void)
{
	return;
}	

void display_driver(void)
{
	return;
}	

void display_file(void)
{
	return;
}	

void display_name(void)
{
	return;
}	

void display_status(void)
{
	pipe_int_write(MOD_PAT);
	pipe_int_write(cur_mod.sngpos);
	pipe_int_write(MOD_POS);
	pipe_int_write(cur_mod.patpos);
}

void display_pausebanner(void)
{
	return;
}	

void display_extractbanner(void)
{
	return;
}	

void display_loadbanner(void)
{
	return;
}	

void display_error(char *myerr, char *myerr_file)
{

}	

void display_driver_error(char *myerr)
{
}	

void display_all(void)
{
	if(quiet) return;
	display_version();
	display_driver();
	display_file();
	display_name();
	display_status();
}	

void update_driver_display(void)
{

}

void update_file_display(void)
{
	return;
}

void update_name_display()
{
	sprintf(cur_mod.name_type,"Name: %s\n"
		"Type: %s, "
		"Periods: %s,%s\n",
		cur_mod.songname==NULL?"(null)":cur_mod.songname,
		cur_mod.modtype,
		(cur_mod.flags&UF_XMPERIODS) ? "XM type" : "mod type",
		(cur_mod.flags&UF_LINEAR) ? "Linear" : "Log");
}

void update_status_display()
{
	return;
}

void exit_display()
{
	return;
}

int current_pattern;
int count_pattern, count_song;

#include <time.h>

int 
get_ui(void)
{
	struct timeval tv;
	fd_set readfds;
	extern int ui_result;
	int code;
	FD_ZERO(&readfds);
	FD_SET(0, &readfds);
	tv.tv_sec = 0;
	tv.tv_usec = 0;
	if(!select(1, &readfds, NULL, NULL, &tv)) {
		return 2; /* no data are available */
		fprintf(stderr,"get_ui not available data\n");
	}
	ui_result = 2;
	pipe_int_read(&code);
	switch(code) {
		case MOD_NEXT:
			ui_result = UI_NEXT_SONG;
			break;
		case MOD_PAUSE:
			ui_result = UI_PAUSE;
			break;
		case MOD_VOL_UP:
			ui_result = UI_VOL_UP;
			break;
		case MOD_VOL_DOWN:
			ui_result = UI_VOL_DOWN;
			break;
		case MOD_STOP:
			ui_result = UI_QUIT;
		case MOD_SET_POS:
			break;
		case MOD_FORWARD:
			ui_result = UI_JUMP_TO_NEXT_PATTERN;
			break;
		case MOD_BACKWARD:
			ui_result = UI_JUMP_TO_PREV_PATTERN;
			break;
	}
	//fprintf(stderr,"get_ui %d %d\n",ui_result,code);
	return ui_result;
}

void
pipe_int_write(int code)
{
	if(sizeof(code) != write(1, &code, sizeof(code))) {
		fprintf(stderr, "pipe_int_write error\n");;
	}
}

void
pipe_int_read(int *code)
{
	if(sizeof(int) != read(0, code, sizeof(int))) {
		fprintf(stderr, "pipe_int_read error\n");
	}
}

void
pipe_string_write(char *s)
{
	int len = strlen(s);
	pipe_int_write(len);
	write(1, s, len);
}

void
pipe_string_read(char *s)
{
	int len;
	pipe_int_read(&len);
	read(0, s, len);
	s[len] = 0;
}
