
/* this file is a part of sori software, (C) Hwang chi-duck 1997               */

#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "sori.h"
#include "mod_c.h"
#include "timidity_c.h"

static float song_pos = 0;
static float pat_pos = 0;
static float num_pat =0;

extern int pipe_error;
void
mod_handle_input(void)
{
	int code, value;
	timidity_int_read(&code);
	switch(code) {
		case MOD_POS:
			timidity_int_read(&value);
			song_pos = value;
			show_position(100*(pat_pos + song_pos/64)/num_pat);
			break;
		case MOD_PAT:
			timidity_int_read(&value);
			pat_pos = value;
			break;
		case MOD_NUM_PAT:
			timidity_int_read(&value);
			num_pat = value;
			break;
		case MOD_END:
			play_end();
			break;
		case MOD_FAILURE:
			fprintf(stderr,"failure to play song\n");
			play_end();
			break;
		default:
			printf("unknown from mod %d\n",code);
			break;
	}
}

void
mod_play_song(char *songname)
{
	int value = -1;
	//printf("mod_play %s\n",songname);
	int fd = open(songname, O_RDONLY);
	if(fd == -1) {
		gdk_beep();
		g_warning("can't open %s", songname);
		return;
	}
	close(fd);

	pipe_error = FALSE;
	timidity_int_write(MOD_NEXT);
	do {
		timidity_int_read(&value);
	} while(value != MOD_NEXT && (!pipe_error));

	timidity_int_write(MOD_PLAY);
	timidity_string_write(songname);
}

void
mod_pause(void)
{
	timidity_int_write(MOD_PAUSE);
}

void
mod_quit(void)
{
	int code;
	timidity_int_write(MOD_NEXT);
	pipe_error = FALSE;
	do {
		timidity_int_read(&code);
	} while(code != MOD_NEXT && (!pipe_error));
	timidity_int_write(MOD_STOP);
	do {
		timidity_int_read(&code);
	} while(code != MOD_STOP && (!pipe_error));
}

void
mod_change_position(float percent)
{
	int new_pat = (int)(percent/100*num_pat);
	int i;
	if(new_pat > pat_pos) {
		i = new_pat - pat_pos;
		while(i-->0) timidity_int_write(MOD_FORWARD);
		return;
	} else if(new_pat < pat_pos) {
		i = pat_pos - new_pat;
		while(i-->0) timidity_int_write(MOD_BACKWARD);
	}
}

void
mod_forward(void)
{
	timidity_int_write(MOD_FORWARD);
}

void
mod_backward(void)
{
	timidity_int_write(MOD_BACKWARD);
}

void
mod_button_callback(GtkWidget *w, gpointer user_data)
{

}
