
/* this file is a part of sori software, (C) Hwang chi-duck 1997               */

#include <sys/stat.h>
#include <unistd.h>
#include <math.h>
#include <stdio.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <fcntl.h>

#include "mp3_p.h"
#include "sori.h"

extern int fpip_in, fpip_out;

off_t song_length;

typedef struct {
	int buffer;
	int playahead;
	int realtime;
} Mp3Config;
extern Mp3Config mp3config;

void
send_msg(PControlMsg msg)
{
	write(fpip_out, msg, sizeof(TControlMsg));
}

int
get_msg(PControlMsg msg)
{
	return read(fpip_in, msg, sizeof(TControlMsg));
}

void
mp3_play_init(void)
{
	TControlMsg msg;
	msg.type = MSG_BUFFER;
	msg.data = mp3config.buffer;
	send_msg(&msg);
	msg.type = MSG_BUFAHEAD;
	msg.data = mp3config.playahead;
	send_msg(&msg);
	msg.type = MSG_PRIORITY;
	msg.data = mp3config.realtime;
	send_msg(&msg);
}

int
mp3_send_query(PControlMsg msg)
{
	TControlMsg rmsg;
	send_msg(msg);
	while(1) {
		get_msg(&rmsg);
		if(rmsg.type == MSG_RESPONSE) break;
	}
	return rmsg.data;
}

int
mp3_is_paused(void)
{
	TControlMsg msg;
	msg.type = MSG_QUERY;
	msg.data = QUERY_PAUSED;
	return mp3_send_query(&msg);
}

void
mp3_pause(void)
{
	TControlMsg msg;
	msg.type = MSG_CTRL;
	msg.data = PLAY_PAUSE;
	send_msg(&msg);
}

void
mp3_play_song(char *filename)
{
	TControlMsg msg;
	struct msghdr hdr;
	struct m_cmsghdr fdhdr;
	struct iovec iov[1];
	char data[2];
	int ctrl = 1 | 2;
	int fd;
	struct stat statbuf;

	fd = open(filename, O_RDONLY);
	if(fd == -1) {
		gdk_beep();
		g_warning("can't open %s", filename);
		return;
	}
	fstat(fd, &statbuf);
	song_length = statbuf.st_size;
	
	msg.type = MSG_SONG;
	msg.data = 0; /* MPEG */

	send_msg(&msg);

	data[0] = ctrl & 1; // Seek backward
	data[1] = ctrl & 2; // Seek forward

	iov[0].iov_base = data;
	iov[0].iov_len = 2;

	hdr.msg_iov = iov;
	hdr.msg_iovlen = 1;
	hdr.msg_name = NULL;
	hdr.msg_namelen = 0;

	fdhdr.cmsg_len = sizeof(struct m_cmsghdr);
	fdhdr.cmsg_level = SOL_SOCKET;
	fdhdr.cmsg_type = SCM_RIGHTS;
	fdhdr.fd = fd;

	hdr.msg_control = &fdhdr;
	hdr.msg_controllen = sizeof(struct m_cmsghdr);

	sendmsg(fpip_out, &hdr, 0);

	close(fd);
}

void
mp3_quit(void)
{
	TControlMsg msg;
	msg.type = MSG_CTRL;
	msg.data = PLAY_STOP;
	send_msg(&msg);
	get_msg(&msg);
}

void
mp3_handle_input(void)
{
	TControlMsg msg;
	get_msg(&msg);
	switch(msg.type) {
		case MSG_NEXT:
			play_next();
			break;
		case MSG_FRAMES:
			//show_play_time((msg.data*32)/1225);
			break;
		case MSG_AUDIOFAILURE:
			break;
		case MSG_POSITION:
			show_position(100.0*msg.data/song_length);
			break;
		default:
			break;
	}
}

void
mp3_change_position(float percent)
{
	TControlMsg msg;
	int pos = (int)floor(percent*song_length/100);

	msg.type = MSG_SEEK;
	msg.data = pos;
	send_msg(&msg);
}

void
mp3_forward()
{
	TControlMsg msg;
	msg.type = MSG_CTRL;
	msg.data = FORWARD_BEGIN;
	send_msg(&msg);
}

void
mp3_backward(void)
{
	TControlMsg msg;
	msg.type = MSG_CTRL;
	msg.data = REWIND_BEGIN;
	send_msg(&msg);
}


