
/* this file is a part of sori software, (C) Hwang chi-duck 1997               */

#include <gtk/gtk.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <ctype.h>
#include "gif.h"
#include "config.h"



typedef struct {
	char *s;
	void (*parser)(FILE *fp, char *s, SoriConfig *config);
} Item;

void general_set(FILE *fp, char *s, SoriConfig *config);
void global_label_set(FILE *fp, char *s, SoriConfig *config);
void global_button_set(FILE *fp, char *s, SoriConfig *config);
void global_gauge_set(FILE *fp, char *s, SoriConfig *config);
void each_label_set(FILE *fp, char *s, SoriConfig *config);
void each_button_set(FILE *fp, char *s, SoriConfig *config);
void each_gauge_set(FILE *fp, char *s, SoriConfig *config);
char *read_line(FILE *fp);
GdkColor * get_color(char *s);
GdkPixmap *get_pixmap(char *s, GdkBitmap **mask);


SoriConfig *
parse_file(char *filename)
{
	char *s;
	char fullname[256];
	FILE *fp;
	int i, n;
	SoriConfig *config;

	Item parse_items[] = {
		{"[General]", general_set}, 
		{"[Global Label Settings]", global_label_set},
		{"[Global Button Settings]", global_button_set},
		{"[Global Gauge Settings]", global_gauge_set},
		{"[Label ", each_label_set},
		{"[Gauge ", each_gauge_set},
		{"[Button ", each_button_set}
	};

	n = sizeof(parse_items)/sizeof(parse_items[0]);

	fp = fopen(filename, "r");
	if(fp == NULL) {
		sprintf(fullname, "%s/.sori/%s", getenv("HOME"), filename);
		fp = fopen(fullname, "r");
	}
	if(fp == NULL) {
		sprintf(fullname, "%s/%s", DATA_DIR, filename);
		fp = fopen(fullname, "r");
	}
	if(fp == NULL) {
		g_warning("read error '%s'\n", filename);
		return NULL;
	}

	config = g_new0(SoriConfig, 1);
	while((s = read_line(fp)) != NULL) {
		for(i=0;i<n;i++) {
			if(strncmp(s, parse_items[i].s, strlen(parse_items[i].s)) == 0) {
				parse_items[i].parser(fp, s, config);
				break;
			}
		}
	}
	return config;
}

void
general_set(FILE *fp, char *s, SoriConfig *config)
{
	int n, i;
	//g_print("%s\n", __FUNCTION__);
	while((s = read_line(fp)) != NULL) {
		if(strncmp(s, "NameOfTheDesigner=", 18) == 0) {
			config->global.designer = g_strdup(s+18);
		} else if(strncmp(s, "WindowHeight=", 13) == 0) {
			config->global.height = atoi(s+13);
		} else if(strncmp(s, "WindowWidth=", 12) == 0) {
			config->global.width = atoi(s+12);
		} else if(strncmp(s, "BackgroundMode=", 15) == 0) {
			s += 15;
			if(strncmp(s, "Gradient", 8) == 0) {
				config->global.background_mode = GradientColor;
			} else if(strncmp(s, "BitmapCenter", 12) == 0) {
				config->global.background_mode = BitmapCenter;
			}
		} else if(strncmp(s, "GradientColorFrom=", 18) == 0) {
			s += 18;
			config->global.gradient_color_from = get_color(s);
		} else if(strncmp(s, "GradientColorTo=", 16) == 0) {
			s += 16;
			config->global.gradient_color_to = get_color(s);
		} else if(strncmp(s, "Bitmap=", 7) == 0) {
			//g_print("Bitmap=\n");
			config->global.pixmap = get_pixmap(s + 7, NULL);
		} else if(strncmp(s, "HideWindowBorder=", 17) == 0) {
			config->global.hide_window_border = atoi(s+17);
		} else if(strncmp(s, "UseWindowShape=", 15) == 0) {
			config->global.use_window_shape = atoi(s+15);
		} else if(strncmp(s, "NumberOfWindowShapePoints=", 26) == 0) {
			n = config->global.number_of_shape_points = atoi(s+26);
			config->global.shape_points = g_new(GdkPoint, n);
			for(i=0;i<n;i++) {
				s = read_line(fp);
				s = strchr(s, '=') + 1;
				config->global.shape_points[i].x = atoi(s);
				s = read_line(fp);
				s = strchr(s, '=') + 1;
				config->global.shape_points[i].y = atoi(s);
			}
		} else if(strncmp(s, "----------", 10) == 0) {
			break;
		} else if(*s == 0) {
		} else if(strncmp(s, "   ", 3) == 0) {
		} else {
			g_warning("invalid global setting %s\n", s);
		}
	}
}

void
global_label_set(FILE *fp, char *s, SoriConfig *config)
{
	//g_print("%s\n", __FUNCTION__);
	while((s = read_line(fp)) != NULL) {
		if(*s == 0) break;
		if(strncmp(s, "FontColor=", 10) == 0) {
		}
	}


}

void
global_gauge_set(FILE *fp, char *s, SoriConfig *config)
{
	//g_print("%s\n", __FUNCTION__);
	while((s=read_line(fp)) != NULL) {
		if(*s == 0) break;
		if(strncmp(s, "ShowShadow=", 11) == 0) {
			config->gauge.show_shadow = atoi(s+11);
		} else if(strncmp(s, "ShadowRelPosX", 13) == 0) {
			config->gauge.shadow_rel_pos_x = atoi(s+13);
		} else if(strncmp(s, "ShadowRelPosY", 13) == 0) {
			config->gauge.shadow_rel_pos_y = atoi(s+13);
		} else if(strncmp(s, "ShadowColor", 11) == 0) {
			config->gauge.shadow_color = get_color(s+11);
		}
	}
}

void
global_button_set(FILE *fp, char *s, SoriConfig *config)
{
	//g_print("%s\n", __FUNCTION__);
	while((s=read_line(fp)) != NULL) {
		if(*s == 0) break;
		if(strncmp(s, "FontColor=", 10) == 0) {
			config->button.font_color= get_color(s+10);
		} else if(strncmp(s, "FontName=", 9) == 0) {
			config->button.font_name = g_strdup(s+9);
			config->button.font = gdk_font_load(config->button.font_name);
			if(config->button.font == NULL)
				config->button.font = gdk_font_load("-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-*-*");
		} else if(strncmp(s, "FontSize=", 9) == 0) {
			config->button.font_size = atoi(s+9);
		} else if(strncmp(s, "FontBold=", 9) == 0) {
			config->button.font_bold = atoi(s+9);
		} else if(strncmp(s, "FontItalic=", 11) == 0) {
			config->button.font_italic = atoi(s+11);
		} else if(strncmp(s, "ShowBevel=", 10) == 0) {
			config->button.show_bevel = atoi(s+10);
		}
	}
}

void
each_label_set(FILE *fp, char *s, SoriConfig *config)
{
	LabelInfo *label = NULL;
	//g_print("%s\n", __FUNCTION__);
	s += 7;
	if(strncmp(s, "YampLogo]", 9) == 0) {
		label = &config->sori_logo_label;
	} else if(strncmp(s, "SongName]", 9) == 0) {
		label = &config->song_name_label;
	} else {
		g_warning("unknown label %s\n", s);
	}
	while((s = read_line(fp)) != NULL) {
		if(*s == 0) break; // empty line: separator
		if(strncmp(s, "Left=", 5) == 0) {
			label->left = atoi(s+5);
		} else if(strncmp(s, "Top=", 4) == 0) {
			label->top = atoi(s+4);
		} else if(strncmp(s, "Width=", 6) == 0) {
			label->width = atoi(s+6);
		} else if(strncmp(s, "Height=", 7) == 0) {
			label->height = atoi(s+7);
		} else if(strncmp(s, "FontColor=", 10) == 0) {
			label->font_color = get_color(s+10);
		} else if(strncmp(s, "FontName=", 9) == 0) {
			label->font_name = g_strdup(s+9);
		} else if(strncmp(s, "FontSize=", 9) == 0) {
			label->font_size = atoi(s+9);
		} else if(strncmp(s, "FontBold=", 9) == 0) {
			label->font_bold = atoi(s+9);
		} else if(strncmp(s, "FontItalic=", 11) == 0) {
			label->font_italic = atoi(s+11);
		} else {
			g_warning("invalid property %s\n", s);
		}
	}
}

void
each_gauge_set(FILE *fp, char *s, SoriConfig *config)
{
	GaugeInfo *gauge = NULL;

	//g_print("%s\n", __FUNCTION__);
	s += 7;
	if(strncmp(s, "SongPos", 7) == 0) {
		gauge = &config->song_pos_gauge;
	} else if(strncmp(s, "PlaylistPos]", 12) == 0) {
		gauge = &config->play_list_pos_gauge;
	} else {
		g_warning("invalid gauge %s\n", s);
	}
	while((s = read_line(fp)) != 0) {
		if(*s == 0) break;
		if(strncmp(s, "Left=", 5) == 0) {
			gauge->left = atoi(s+5);
		} else if(strncmp(s, "Top=", 4) == 0) {
			gauge->top = atoi(s+4);
		} else if(strncmp(s, "Height=", 7) == 0) {
			gauge->height = atoi(s+7);
		} else if(strncmp(s, "Width=", 6) == 0) {
			gauge->width = atoi(s+6);
		} else {
			g_warning("invalid gauge property %s\n", s);
		}
	}
}

void
each_button_set(FILE *fp, char *s, SoriConfig *config)
{
	ButtonInfo *button = NULL;
	s += 8;
	if(strncmp(s, "Minimize]", 9) == 0) {
		button = &config->minimize_button; 
	} else if(strncmp(s, "Close]", 6) == 0) {
		button = &config->close_button;
	} else if(strncmp(s, "Next Song]", 10) == 0) {
		button = &config->next_song_button;
	} else if(strncmp(s, "Previous Song]", 14) == 0) {
		button = &config->previous_song_button;
	} else if(strncmp(s, "Fast Forward]", 13) == 0) {
		button = &config->fast_forward_button;
	} else if(strncmp(s, "Rewind]", 7) == 0) {
		button = &config->rewind_button;
	} else if(strncmp(s, "Help]", 5) == 0) {
		button = &config->help_button;
	} else if(strncmp(s, "About]", 6) == 0) {
		button = &config->about_button;
	} else if(strncmp(s, "Options]", 7) == 0) {
		button = &config->options_button;
	} else if(strncmp(s, "Disk Operations]", 15) == 0) {
		button = &config->disk_operations_button;
	} else if(strncmp(s, "Song Information]", 16) == 0) {
		button = &config->song_information_button;
	} else if(strncmp(s, "Pause]", 6) == 0) {
		button = &config->pause_button;
	} else if(strncmp(s, "Stop]", 5) == 0) {
		button = &config->stop_button;
	} else if(strncmp(s, "Play]", 5) == 0) {
		button = &config->play_button;
	} else {
		g_warning("unknown button %s\n", s);
	}
	while((s = read_line(fp)) != NULL) {
		if(*s == 0) break; // empty line: separator
		if(strncmp(s, "Bitmap1=", 8) == 0) {
			button->pixmap1 = get_pixmap(s + 8, &button->mask1);
			//button->bitmap1_name = g_strdup(s+8);
		} else if(strncmp(s, "Bitmap2=", 8) == 0) {
			button->pixmap2 = get_pixmap(s + 8, &button->mask2);
			//button->bitmap2_name = g_strdup(s+8);
		} else if(strncmp(s, "Left=", 5) == 0) {
			button->left = atoi(s + 5);
		} else if(strncmp(s, "Top=", 4) == 0) {
			button->top = atoi(s + 4);
		} else if(strncmp(s, "Height=", 7) == 0) {
			button->height = atoi(s+7);
		} else if(strncmp(s, "Width=", 6) == 0) {
			button->width = atoi(s+6);
		} else if(strncmp(s, "Caption=", 8) == 0) {
			button->caption = g_strdup(s+8);
		} else {
			g_warning("unknown property %s in button setting\n", s);
		}
	}
}

char *
read_line(FILE *fp)
{
	static char buf[256];
	char *s;
	int len;
	s = fgets(buf, 256, fp);
	if(s == NULL) return NULL;
	len = strlen(buf) - 1;
	if(buf[len] == '\n') buf[len--] = 0;
	if(buf[len] == '\r') buf[len--] = 0;
	return buf;
}

GdkColor *
get_color(char *s)
{
	char name[20];
	GdkColor *color;
	color = g_new(GdkColor, 1);
	if(*s == '$') {
		strcpy(name, "rgb:");
		name[4] = s[1];
		name[5] = s[2];
		name[6] = '/';
		name[7] = s[3];
		name[8] = s[4];
		name[9] = '/';
		name[10] = s[5];
		name[11] = s[6];
		name[12] = 0;
		gdk_color_parse(name, color);
		gdk_color_alloc(gdk_colormap_get_system(), color);
	} else if(*s == '0') {
		color->red = color->green = color->blue = 0;
		gdk_color_alloc(gdk_colormap_get_system(), color);
	}

	return color;
}

GdkPixmap *
get_pixmap(char *filename, GdkBitmap **mask)
{
	GdkPixmap *pixmap;
	int i;
	char fullname[256];
	GdkColor color;
	//g_print("%s: %s\n", __FUNCTION__, s);
	if(strcmp(".gif", filename + strlen(filename) - 4) != 0) {
		g_warning("not gif: %s\n", filename);
		return NULL;
	}
	color.pixel = 0;
	for(i=0;filename[i];i++) {
		filename[i] = tolower(filename[i]);
		if (filename[i] == '\\') filename[i] = '/';
	}
	sprintf(fullname, "%s/.sori/%s", getenv("HOME"), filename);
	if (access(fullname, R_OK)) {
		sprintf(fullname, "%s/%s", DATA_DIR, filename);
		if (access(fullname, R_OK)) {
			g_warning("can't find the file %s\n", filename);
			return NULL;
		}
	}

	pixmap = gdk_pixmap_create_from_gif(NULL, mask, &color, fullname);
	return pixmap;
}

