
/* this file is a part of sori software, (C) Hwang chi-duck 1997               */

#include "sori.h"

#include "space.h"
#include "button.h"
#include "sorilabel.h"
#include "gauge.h"
#include <gdk/gdkprivate.h>

void create_gradient_pixmap(SoriConfig *config);

GtkWidget * button_create(GtkWidget *space, void (*cb)(GtkWidget *), ButtonInfo*);
GtkWidget * label_create(GtkWidget *space, SoriConfig *, LabelInfo *);
GtkWidget * gauge_create(GtkWidget *space, SoriConfig *, GaugeInfo *);
void quick_design_list(GtkWidget *);
void quick_play_list(GtkWidget *);
void raise_window(GtkWidget *w);
void window_clip_and_set_pixmap(GtkWidget *widget, int width, int height, GdkPixmap *pixmap, GdkPoint *points, int npoints);
void do_background(GtkWidget *w, SoriConfig *config);


SoriConfig *config;

void
sori_destroy(SoriConfig *config)
{
}


GtkWidget *
sori_new(char *filename)
{
	GtkWidget *main_win, *space;
	GtkTooltips *tooltips;
	GtkWidget *minimize_button, *close_button;
	GtkWidget *next_song_button, *previous_song_button;
	GtkWidget *fast_forward_button, *rewind_button;
	GtkWidget *help_button, *about_button;
	GtkWidget *options_button, *disk_operations_button;
	GtkWidget *song_information_button;
	GtkWidget *pause_button, *stop_button, *play_button;

	GtkWidget *sori_logo_label;
	
	config = parse_file(filename);
	if(config == NULL) return NULL;

	if(config->global.background_mode == GradientColor) {
		create_gradient_pixmap(config);
	}
	// main window
	//g_print("gtk_shape\n");
	main_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_events(main_win, gtk_widget_get_events(main_win) | GDK_BUTTON_PRESS_MASK);
	gtk_signal_connect(GTK_OBJECT(main_win), "realize",
		GTK_SIGNAL_FUNC(do_background), config);
	//gtk_shape_window_new(config->global.pixmap, 
	
	//config->global.shape_points, config->global.number_of_shape_points);
	gtk_window_set_title(GTK_WINDOW(main_win), "sori");
	gtk_signal_connect(GTK_OBJECT(main_win), "button_press_event",
		GTK_SIGNAL_FUNC(raise_window), NULL);
	space = gtk_space_new(config->global.width, config->global.height);
	//g_print("gtk_space_new\n");

	gtk_container_add(GTK_CONTAINER(main_win), space);
	gtk_widget_show(space);
	gtk_widget_realize(space);
	gtk_widget_realize(main_win);



	// label

	// song name
	song_name_label = label_create(space, config, &config->song_name_label);
	gtk_sori_label_set(GTK_SORI_LABEL(song_name_label), "empty list");

	// logo
	sori_logo_label = label_create(space, config, &config->sori_logo_label);
	gtk_sori_label_set(GTK_SORI_LABEL(sori_logo_label), "SORI");

/*==========================================================================*/
	// button

	tooltips = gtk_tooltips_new();

	//minimize
	minimize_button = button_create(space, minimize_cb,&config->minimize_button);

	//close
	close_button = button_create(space, close_cb, &config->close_button);
	gtk_tooltips_set_tips(tooltips, close_button, "Quit");

	//next song
	next_song_button = button_create(space, next_song_cb, &config->next_song_button);
	gtk_tooltips_set_tips(tooltips, next_song_button, "Next Song");

	//previous song
	previous_song_button = button_create(space, previous_song_cb, &config->previous_song_button);
	gtk_tooltips_set_tips(tooltips, previous_song_button, "Previous Song");


	//fast forward
	fast_forward_button = button_create(space, fast_forward_cb, &config->fast_forward_button);
	gtk_tooltips_set_tips(tooltips, fast_forward_button, "Fast Forward");

	//rewind
	rewind_button = button_create(space, rewind_cb, &config->rewind_button);
	gtk_tooltips_set_tips(tooltips, rewind_button, "Rewind");

	//help
	help_button = button_create(space, help_cb, &config->help_button);
	gtk_tooltips_set_tips(tooltips, help_button, "Help");

	//about
	about_button = button_create(space, about_cb, &config->about_button);
	gtk_tooltips_set_tips(tooltips, about_button, "About");

	//options
	options_button = button_create(space, options_cb, &config->options_button);
	gtk_tooltips_set_tips(tooltips, options_button, "Volume & Play Mode");
	gtk_signal_connect(GTK_OBJECT(options_button), "right_clicked",
		GTK_SIGNAL_FUNC(quick_design_list), NULL);

	//disk operations
	disk_operations_button = button_create(space, disk_operations_cb, &config->disk_operations_button);
	gtk_tooltips_set_tips(tooltips, disk_operations_button, "Load Songs");

	// song information
	song_information_button = button_create(space, song_information_cb, &config->song_information_button);
	gtk_tooltips_set_tips(tooltips, song_information_button, "Song Informations");

	//pause
	pause_button = button_create(space, pause_cb, &config->pause_button);
	gtk_tooltips_set_tips(tooltips, pause_button, "Pause");

	//stop
	stop_button = button_create(space, stop_cb, &config->stop_button);
	gtk_tooltips_set_tips(tooltips, stop_button, "Stop");

	//play
	play_button = button_create(space, play_cb, &config->play_button);
	gtk_tooltips_set_tips(tooltips, play_button, "Play");
	gtk_signal_connect(GTK_OBJECT(play_button), "right_clicked",
		GTK_SIGNAL_FUNC(quick_play_list), NULL);

/*========================================================================*/
	// gauge

	// song position
	song_pos_gauge = gauge_create(space, config, &config->song_pos_gauge);
	gtk_signal_connect(GTK_OBJECT(song_pos_gauge), "pressed",
		GTK_SIGNAL_FUNC(song_position_change_cb), NULL);
	gtk_tooltips_set_tips(tooltips, song_pos_gauge, "Song Position");

	// play list position
	play_list_pos_gauge = gauge_create(space, config, &config->play_list_pos_gauge);
	gtk_signal_connect(GTK_OBJECT(play_list_pos_gauge), "pressed",
		GTK_SIGNAL_FUNC(play_list_position_cb), NULL);
	gtk_tooltips_set_tips(tooltips, play_list_pos_gauge, "Play List Position");

	gtk_widget_show(main_win);
	return main_win;
}

GtkWidget *
button_create(GtkWidget *space, void (*callback)(GtkWidget *w), ButtonInfo *buttoninfo)
{
	GtkWidget *button;
	GdkGC *gc;

	if(buttoninfo->caption) {
		gc = gdk_gc_new(space->parent->window);
		gdk_gc_set_foreground(gc, config->button.font_color);
		button = gtk_sori_button_new_with_label(buttoninfo->caption, gc, config->button.font, buttoninfo->width, buttoninfo->height);
	} else {
		button = gtk_sori_button_new_with_pixmap(buttoninfo->pixmap1,
			buttoninfo->mask1, buttoninfo->pixmap2, buttoninfo->mask2, buttoninfo->width, buttoninfo->height);
	}
	gtk_space_put(GTK_SPACE(space), button, buttoninfo->left, buttoninfo->top);
	//g_print("%d %d\n", buttoninfo->left, buttoninfo->top);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(callback), NULL);
	gtk_widget_show(button);
	return button;
}

GtkWidget *
label_create(GtkWidget *space, SoriConfig *config, LabelInfo *labelinfo)
{
	GtkWidget *label = NULL;
	GdkGC *gc;
	GdkFont *font;

	gc = gdk_gc_new(space->parent->window);
	if(labelinfo->font_color) gdk_gc_set_foreground(gc, labelinfo->font_color);
	else ; //gdk_gc_set_foreground(gc, config->label.font_color);
	font = gdk_font_load("-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-*-*");
	gdk_gc_set_font(gc, font);
	label = gtk_sori_label_new(gc, font, "label", labelinfo->width, labelinfo->height);
	//g_print("%s %d %d\n", __FUNCTION__, labelinfo->width, labelinfo->height);
	gtk_space_put(GTK_SPACE(space), label, labelinfo->left, labelinfo->top);
	gtk_widget_show(label);
	return label;
}

GtkWidget *
gauge_create(GtkWidget *space, SoriConfig *config, GaugeInfo *gaugeinfo)
{
	GtkWidget *gauge = NULL;
	gauge = gtk_gauge_new(gaugeinfo->width, gaugeinfo->height, NULL);
	//g_print("======%d %d\n", gaugeinfo->width, gaugeinfo->height);

	gtk_space_put(GTK_SPACE(space), gauge, gaugeinfo->left, gaugeinfo->top);
	gtk_widget_show((GtkWidget *)gauge);
	return gauge;
}


void 
create_gradient_pixmap(SoriConfig *config)
{
	GdkPixmap *pixmap;
	GdkVisual *visual = gdk_visual_get_system();
	GdkColormap *cmap;
	int width = config->global.width;
	int height = config->global.height;
	int i;
	double fraction;
	GdkGC *gc;
	GdkColor *color;
	GdkColor *from_color, *to_color;

	cmap = gdk_colormap_get_system();
	from_color = config->global.gradient_color_from;
	to_color = config->global.gradient_color_to;
	color = g_new(GdkColor, 1);

	pixmap = gdk_pixmap_new (NULL, width, height, visual->depth);
	gc = gdk_gc_new(pixmap);
	for(i=0;i<height;i++) {
		fraction = ((double)i)/(height - 1);
		color->red = from_color->red*(1.0 - fraction) + fraction*to_color->red;
		color->green = from_color->green*(1.0 - fraction) + fraction*to_color->green;
		color->blue = from_color->blue*(1.0 - fraction) + fraction*to_color->blue;
		gdk_color_alloc(cmap, color);
		gdk_gc_set_foreground(gc, color);
		gdk_draw_line(pixmap, gc, 0, i, width - 1, i);
	}
	gdk_gc_destroy(gc);
	g_free(color);
	config->global.pixmap = pixmap;
}

void
quick_design_list(GtkWidget *w)
{
	//g_print("%s\n", __FUNCTION__);
}

void
quick_play_list(GtkWidget *w)
{
	//g_print("%s\n", __FUNCTION__);
}

void 
raise_window(GtkWidget *w)
{
	gdk_window_raise(w->window);
}

void
do_background(GtkWidget *widget, SoriConfig *config)
{
	window_clip_and_set_pixmap(widget, config->global.width, config->global.height, config->global.pixmap, config->global.shape_points, config->global.number_of_shape_points); 
}

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/shape.h>

void
window_clip_and_set_pixmap(GtkWidget *widget, int width, int height, GdkPixmap *pixmap, GdkPoint *points, int npoints)
{
	GdkBitmap *mask;
	GdkWindowPrivate * private;
	GdkGC *gc;
	GdkColor tmp_color;
	gdk_window_set_back_pixmap(widget->window, pixmap, FALSE);
	private = (GdkWindowPrivate *)widget->window;
	mask = gdk_pixmap_new (NULL, width, height, 1);
	gc = gdk_gc_new(mask);
	tmp_color.pixel = 0;
	gdk_gc_set_foreground (gc, &tmp_color);
	gdk_draw_rectangle (mask, gc, TRUE, 0, 0, -1, -1);
	tmp_color.pixel = ~((gulong) 0);
	gdk_gc_set_foreground (gc, &tmp_color);
	gdk_draw_polygon(mask, gc, TRUE, points, npoints);
	XShapeCombineMask(private->xdisplay,private->xwindow , ShapeBounding, 0, 0, ((GdkWindowPrivate *)mask)->xwindow, ShapeSet);
	gdk_gc_destroy(gc);
	gdk_pixmap_destroy(mask);
}
