
/* this file is a part of sori software, (C) Hwang chi-duck 1997               */

/*
 *
 *  α׷ ۱ Ȳġ ϴ.
 *  α׷ GNU GENERAL PUBLIC LICENSE ǰ ˴ϴ.
 *  α׷   ü, ,
 * ȸ س  ȭ  å ϴ.
 * 
 */


#include <unistd.h>
#include <fnmatch.h>   // fnmatch
#include <sys/types.h> // kill
#include <signal.h>    // kill
#include <sys/wait.h>  // wait
#include <string.h>    // strrchr
#include <time.h>      // time
#include <sys/socket.h> // socketpair
extern char *basename(char *);

#include "sorilabel.h"
#include "gauge.h"


#include "sori.h"
#include "dialog.h"

#include "timidity_c.h"
#include "mp3_c.h"
#include "mod_c.h"

typedef struct {
	int buffer;
	int playahead;
	int realtime;
} Mp3Config;

Mp3Config mp3config = { 20, 5, 0};


GtkWidget *song_name_label;
GtkWidget *song_pos_gauge;
GtkWidget *play_list_pos_gauge;

int song_type = -1;
int pid = -2;
int not_play = TRUE;
int player_dead = TRUE;
int input_tag = -1;
int fpip_in = -1;
int fpip_out = -1;
int auto_next = TRUE;
int shuffle = FALSE;
int paused = FALSE;
int pipeAppli[2], pipePlayer[2];/* Pipe for communication with Music Player */

char midiplayer[] = "timidity -il -p 20 foo";
char mp3player[] = "sajberplay";
char modplayer[] = "mod";

GtkWidget *main_win = NULL;


void handle_input(gpointer data, int source, GdkInputCondition condition);
void child_signal_handler(int signum);
void usr1_signal_handler(int signum);
int init_list(void);


int
main(int argc, char *argv[])
{
	char rc_fullname[256];
	//if(fork()) exit(0);
	gtk_init(&argc, &argv);

	//chdir(getenv("HOME"));
	//chdir(".sori");
	sprintf(rc_fullname, "%s/.sori/gtkrc", getenv("HOME"));
	gtk_rc_parse(rc_fullname);
	signal(SIGCHLD, child_signal_handler);
	signal(SIGUSR1, usr1_signal_handler);

	if (argc > 1) 
		main_win = sori_new(argv[1]);
	else 
		main_win = sori_new("XeReX standard.design");
	if (main_win == NULL) return 1;

	//gtk_timeout_add(2000, init_list, NULL);
	srand(time(NULL));

	gtk_main();
	default_list_save();
	return 0;
}

gint 
init_list(void)
{
	extern GtkWidget *fs_dialog;
	if (fs_dialog) return FALSE;
	fs_dialog = fs_dialog_create();
	gtk_widget_show(fs_dialog);
	return FALSE;
}

void 
play_song(char *songname)
{
	int new_song_type;
	
	new_song_type = file_type(songname);

	//play_init();
	//decompress(songname);

	if (new_song_type != song_type) {
		kill_player();
		song_type = new_song_type;
		fork_player();
	}

	switch(song_type) {
		case MIDI:
			timidity_play_song(songname);
			break;
		case MP3:
			mp3_play_song(songname);
			break;
		case MOD:
			mod_play_song(songname);
			break;
		default:
			fprintf(stderr,"I cannot know how to play %s \n",songname);
			fprintf(stderr,"This song will be ignored\n");
			play_next();
			return;
	}
	//ani_start((AniPixmap *)face);
	set_current_song_name(basename(songname));
	not_play = FALSE;
	paused = FALSE;
	gtk_gauge_set_value((GtkGauge*)play_list_pos_gauge, (double)(current_file+1.0)/nfiles);
}

int 
fork_player(void)
{
	char *player;
	char tmpbuf[1024];


	switch(song_type) {
		case MIDI:
			player= midiplayer;
			break;
		case MP3:
			player= mp3player;
			break;
		case MOD:
			player= modplayer;
			break;
		default:
			return 0;
	}
	player_dead = FALSE;

	socketpair(AF_UNIX, SOCK_STREAM, 0, pipeAppli);
	socketpair(AF_UNIX, SOCK_STREAM, 0, pipePlayer);

	if ((pid = fork()) == 0) {	/*child */
		close(pipePlayer[1]);
		close(pipeAppli[0]);
		dup2(pipePlayer[0], 0);
		dup2(pipeAppli[1], 1);
		close(pipePlayer[0]);
		close(pipeAppli[1]);

		strcpy(tmpbuf, player);
		execlp("sh", "sh", "-c", tmpbuf, NULL);
		/* Won't come back from here */
		g_warning("come back from PLAYER %s\n",player);
		_exit(1);
	}
	close(pipeAppli[1]);
	close(pipePlayer[0]);

	fpip_in = pipeAppli[0];
	fpip_out = pipePlayer[1];
	input_tag = gdk_input_add(fpip_in, GDK_INPUT_READ, handle_input, NULL);
	if (song_type == MP3) {
		//GtkAdjustment *adj = GTK_RANGE(locator_scale)->adjustment;
		mp3_play_init();
		//adj->upper = 110;
		//adj->step_increment = 10;
		//adj->value = 0;
	}
	return 1;
}

void
player_is_dead(void)
{
	not_play = TRUE;
	player_dead = TRUE;
	song_type = -1;
	gdk_input_remove(input_tag);
	close(fpip_in);
	close(fpip_out);
}

void
play_next(void)
{
	if (nfiles == 0) {
		current_file = -1;
		return;
	}
	current_file++;
	if (current_file >= nfiles) {
		current_file = 0;
	}

	select_update();
}


int
file_type(char *filename)
{
	struct { char * pattern; int type;} pattern[]=
	{
		{"*.mp3", MP3},
		{"*.mid", MIDI},
		{"*.mid.gz", MIDI},
		{"*.mod", MOD},
		{"*.mod.gz", MOD},
		{"*.s3m", MOD},
		{"*.s3m.gz", MOD},
		{"*.xm",  MOD},
		{"*.xm.gz", MOD},
		{"*.669", MOD},
		{"*.669.gz", MOD},
		{"*.mtm", MOD},
		{"*.mtm.gz", MOD},
	};
	int npat = sizeof(pattern)/sizeof(pattern[0]);
	int i;
	for(i = 0;i< npat;i++) {
		if (fnmatch(pattern[i].pattern, filename, 0) == 0) {
			return pattern[i].type;
		}
	}
	return -1;
}


void
kill_player(void)
{
	switch(song_type) {
		case MIDI:
			timidity_quit();
			//kill(pid, SIGKILL);
			//wait(NULL);
			break;
		case MP3:
			//mp3_quit();
			kill(pid, SIGQUIT);
			kill(pid+1, SIGQUIT);
			kill(pid+2, SIGQUIT);
			usleep(500000);
			break;
		case MOD:
			//mod_quit();
			kill(pid, SIGKILL);
			//wait(NULL);
			break;
		default:
			break;
	}
	if (input_tag > 0) gdk_input_remove(input_tag);
	/*
	wait_dsp_free();
	*/
	while(!player_dead) sleep(1);
}

void
child_signal_handler(int signum)
{
	//g_print("child\n");
	if (wait(NULL) == pid) {
		player_is_dead();
		if (fpip_in > 0) close(fpip_in);
		if (fpip_out > 0) close(fpip_out);
	}
	signal(SIGCHLD, child_signal_handler);
}

void 
set_current_song_name(char *song)
{
	char *songname = g_strdup(basename(song));
	char *s;
	if (fnmatch("*.gz", songname, 0) == 0) {
		songname[strlen(songname) - 3] = 0;
	}
	s = strrchr(songname, '.');
	if (s != NULL) *s = 0;
	gtk_sori_label_set(GTK_SORI_LABEL(song_name_label), songname);
	//gtk_window_set_title(GTK_WINDOW(toplevel), songname);
	g_free(songname);
}

void 
handle_input(gpointer data, int source, GdkInputCondition condition)
{
	switch(song_type) {
		case MIDI:
			timidity_handle_input();
			break;
		case MP3:
			mp3_handle_input();
			break;
		case MOD:
			mod_handle_input();
			break;
		default:
			break;
	}
}

void
play_end(void)
{
	if (auto_next) {
		if (shuffle) current_file = rand()%nfiles;
		play_next();
	} else {
		not_play = TRUE;
	}
}

void
change_position(float percent)
{
	switch(song_type) {
		case MIDI:
			timidity_change_position(percent);
			break;
		case MP3:
			mp3_change_position(percent);
			break;
		case MOD:
			mod_change_position(percent);
			break;
		default:
			break;
	}
}

void
show_position(float percent)
{
	//int old_value = 100 * ((GtkGauge *)song_pos_gauge)->adjustment->value;
	//if (old_value == (int)percent) return;
	//((GtkGauge *)song_pos_gauge)->adjustment->value = percent/100;
	//gtk_widget_draw(song_pos_gauge, NULL);
	gtk_gauge_set_value((GtkGauge*)song_pos_gauge, percent/100);
}

void
play_previous(void)
{
	if (nfiles == 0) {
		current_file = -1;
		return;
	}
	current_file--;
	if (current_file < 0) {
		current_file = nfiles -1;
	}
	select_update();
}

void
usr1_signal_handler(int signum)
{
	if (main_win && !GTK_WIDGET_VISIBLE(main_win)) {
		gtk_widget_show(main_win);
	}
	signal(SIGUSR1, usr1_signal_handler);
}

