
/* this file is a part of sori software, (C) Hwang chi-duck 1997               */

/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <gtk/gtkwidget.h>
#include "sorilabel.h"


static void gtk_sori_label_class_init   (GtkSoriLabelClass  *klass);
static void gtk_sori_label_init         (GtkSoriLabel       *label);
static void gtk_sori_label_destroy      (GtkObject      *object);
static void gtk_sori_label_size_request (GtkWidget      *widget,
				    GtkRequisition *requisition);
void gtk_sori_label_size_allocate (GtkWidget     *widget,
			  GtkAllocation *allocation);
static gint gtk_sori_label_expose       (GtkWidget      *widget,
				    GdkEventExpose *event);
static void
gtk_sori_label_realize (GtkWidget *widget);


static GtkWidgetClass *parent_class = NULL;


guint
gtk_sori_label_get_type ()
{
  static guint label_type = 0;

  if (!label_type)
    {
      GtkTypeInfo label_info =
      {
	"GtkSoriLabel",
	sizeof (GtkSoriLabel),
	sizeof (GtkSoriLabelClass),
	(GtkClassInitFunc) gtk_sori_label_class_init,
	(GtkObjectInitFunc) gtk_sori_label_init,
	(GtkArgFunc) NULL,
      };

      label_type = gtk_type_unique (gtk_widget_get_type (), &label_info);
    }

  return label_type;
}

void
gtk_sori_label_class_init (GtkSoriLabelClass *class)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;

  object_class = (GtkObjectClass*) class;
  widget_class = (GtkWidgetClass*) class;

  parent_class = gtk_type_class (gtk_widget_get_type ());

  object_class->destroy = gtk_sori_label_destroy;

  widget_class->size_request = gtk_sori_label_size_request;
  widget_class->size_allocate = gtk_sori_label_size_allocate;
  widget_class->expose_event = gtk_sori_label_expose;
  widget_class->realize = gtk_sori_label_realize;
}

void
gtk_sori_label_init (GtkSoriLabel *label)
{
  GTK_WIDGET_SET_FLAGS (label, GTK_NO_WINDOW);

  label->label = NULL;
}

GtkWidget*
gtk_sori_label_new (GdkGC *gc, GdkFont *font, char *str, int width, int height)
{
	GtkSoriLabel *label;
  g_return_val_if_fail (str != NULL, NULL);


  label = gtk_type_new (gtk_sori_label_get_type ());
	label->gc = gc;
	label->font = font;
	label->width = width;
	label->height = height;

  gtk_sori_label_set (label, str);

  return GTK_WIDGET (label);
}

void
gtk_sori_label_set (GtkSoriLabel *label,
	       char     *str)
{
	int width;
  g_return_if_fail (label != NULL);
  g_return_if_fail (GTK_IS_SORI_LABEL (label));
  g_return_if_fail (str != NULL);

  if (label->label)
    g_free (label->label);
  label->label = g_strdup (str);
  if((width = gdk_string_width(label->font, label->label)) > label->width) {
	char *s;
	s = label->label + strlen(label->label);
	do {
		s--;
		width -= gdk_char_width(label->font, *s);
	}while(width > label->width);
#if HANGUL
	if(is_second_han(label->label, s - label->label)) s--;
#endif
	*s = 0;
  }



  if (GTK_WIDGET_VISIBLE (label))
    {
      if (GTK_WIDGET_MAPPED (label))
	gdk_window_clear_area (GTK_WIDGET (label)->window,
			       GTK_WIDGET (label)->allocation.x,
			       GTK_WIDGET (label)->allocation.y,
			       GTK_WIDGET (label)->allocation.width,
			       GTK_WIDGET (label)->allocation.height);
	  gtk_sori_label_expose(GTK_WIDGET(label), NULL);
		//gtk_sori_label_size_request (GTK_WIDGET(label), &GTK_WIDGET(label)->requisition);
	   //GTK_WIDGET (label)->allocation.width = GTK_WIDGET(label)->requisition.width;
   //GTK_WIDGET (label)->allocation.height = GTK_WIDGET(label)->requisition.height;

    }
}

void
gtk_sori_label_get (GtkSoriLabel  *label,
	       char     **str)
{
  g_return_if_fail (label != NULL);
  g_return_if_fail (GTK_IS_SORI_LABEL (label));
  g_return_if_fail (str != NULL);

  *str = label->label;
}


static void
gtk_sori_label_destroy (GtkObject *object)
{
  GtkSoriLabel *label;

  g_return_if_fail (object != NULL);
  g_return_if_fail (GTK_IS_SORI_LABEL (object));

  label = GTK_SORI_LABEL (object);

  if (GTK_WIDGET (object)->parent &&
      GTK_WIDGET_MAPPED (object))
    gtk_widget_unmap (GTK_WIDGET (object));

  g_free (label->label);
  gdk_gc_destroy(label->gc);
  gdk_font_free(label->font);

  if (GTK_OBJECT_CLASS (parent_class)->destroy)
    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static void
gtk_sori_label_size_request (GtkWidget      *widget,
			GtkRequisition *requisition)
{
  GtkSoriLabel *label;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_SORI_LABEL (widget));
  g_return_if_fail (requisition != NULL);
	//g_print("%s\n", __FUNCTION__);

  label = GTK_SORI_LABEL (widget);

  requisition->width = label->width;
  requisition->height = label->height;
  //requisition->width = gdk_string_width(label->font, label->label);
  //requisition->height = label->font->descent + label->font->ascent;
}

static gint
gtk_sori_label_expose (GtkWidget      *widget,
		  GdkEventExpose *event)
{
  GtkSoriLabel *label;
  int x, y;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_SORI_LABEL (widget), FALSE);
  if (!GTK_WIDGET_VISIBLE (widget) || !GTK_WIDGET_MAPPED (widget)) return TRUE;
  //g_return_val_if_fail (event != NULL, FALSE);
  label = GTK_SORI_LABEL(widget);
  y = widget->allocation.y + (label->height + label->font->ascent - label->font->descent)/2;
  x = widget->allocation.x + label->width/2 - gdk_string_width(label->font, label->label)/2;
  //requisition->width = gdk_string_width(label->font, label->label);
  //g_print("%s %d %d\n", __FUNCTION__, x, y);
  //g_print("%d %d %d\n", widget->allocation.y, label->height, label->font->ascent);
  gdk_draw_string (widget->window, label->font, label->gc,
			   x, y, label->label);

  return TRUE;
}


static void
gtk_sori_label_realize (GtkWidget *widget)
{

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_SORI_LABEL (widget));
	//g_print("%s\n", __FUNCTION__);

  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

  widget->window = widget->parent->window;
  widget->style = gtk_style_attach (widget->style, widget->window);
}

void
gtk_sori_label_size_allocate (GtkWidget     *widget,
			  GtkAllocation *allocation)
{
	//g_print("%s %d %d\n", __FUNCTION__, allocation->x, allocation->y);
	widget->allocation.x = allocation->x;
	widget->allocation.y = allocation->y;
}
