/* this file is a part of sori software, (C) Hwang chi-duck 1997              */

 
#include "space.h"

static void gtk_space_add(GtkContainer     *container, GtkWidget        *widget);
static void gtk_space_class_init (GtkSpaceClass *class);
static void gtk_space_init (GtkSpace *space);
static void gtk_space_size_allocate (GtkWidget     *widget, GtkAllocation *allocation);
static void gtk_space_size_request (GtkWidget      *widget, GtkRequisition *requisition);
static gint gtk_space_expose (GtkWidget      *widget, GdkEventExpose *event);
static void gtk_space_draw (GtkWidget    *widget, GdkRectangle *area);
static void gtk_space_map (GtkWidget *widget);
static void gtk_space_unmap (GtkWidget *widget);
static void gtk_space_foreach (GtkContainer *container,
		 GtkCallback   callback,
		 gpointer      callback_data);

guint
gtk_space_get_type ()
{
  static guint space_type = 0;

  if (!space_type)
    {
      GtkTypeInfo space_info =
      {
	"GtkSpace",
	sizeof (GtkSpace),
	sizeof (GtkSpaceClass),
	(GtkClassInitFunc) gtk_space_class_init,
	(GtkObjectInitFunc) gtk_space_init,
	(GtkArgFunc) NULL,
      };

      space_type = gtk_type_unique (gtk_container_get_type (), &space_info);
    }

  return space_type;
}

static void
gtk_space_class_init (GtkSpaceClass *class)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  GtkContainerClass *container_class;

  object_class = (GtkObjectClass*) class;
  widget_class = (GtkWidgetClass*) class;
  container_class = (GtkContainerClass*) class;

  widget_class->size_allocate = gtk_space_size_allocate;
  widget_class->size_request = gtk_space_size_request;
  widget_class->expose_event = gtk_space_expose;
  widget_class->draw = gtk_space_draw;
  widget_class->map = gtk_space_map;
  widget_class->unmap = gtk_space_unmap;

  container_class->add = gtk_space_add;
  container_class->foreach = gtk_space_foreach;

}

static void
gtk_space_init (GtkSpace *space)
{
	GTK_WIDGET_SET_FLAGS (space, GTK_NO_WINDOW);
	space->children = NULL;

}

GtkWidget *
gtk_space_new(int width, int height)
{
	GtkWidget *widget;
	
	widget = gtk_type_new(gtk_space_get_type());
	widget->requisition.width = width;
	widget->requisition.height = height;

	return widget;
}

void
gtk_space_put(GtkSpace *space, GtkWidget *child, int x, int y)
{
	GtkSpaceChild *child_info;
	g_return_if_fail( space != NULL);
	g_return_if_fail( child != NULL);
	g_return_if_fail( GTK_IS_SPACE(space));

	child_info = g_new(GtkSpaceChild, 1);
	child_info->widget = child;
	child_info->x = x;
	child_info->y = y;
	space->children = g_list_append(space->children, child_info);
	gtk_widget_set_parent (child, GTK_WIDGET (space));
    if (GTK_WIDGET_VISIBLE (child) && GTK_WIDGET_VISIBLE (space))
      gtk_widget_queue_resize (child);
}

void
gtk_space_add(GtkContainer *space, GtkWidget *w)
{
	g_print("use gtk_space_put\n");
}


static void
gtk_space_foreach (GtkContainer *container,
		 GtkCallback   callback,
		 gpointer      callback_data)
{
	GtkSpace *space;
	GtkSpaceChild *child;
  GList *children;

  g_return_if_fail (container != NULL);
  g_return_if_fail (GTK_IS_SPACE (container));
  g_return_if_fail (callback != NULL);

  space = GTK_SPACE(container);

  children = space->children;
  while (children)
    {
      child = children->data;
      children = children->next;

	(* callback) (child->widget, callback_data);
    }
}

static void
gtk_space_size_allocate (GtkWidget     *widget,
			GtkAllocation *allocation)
{
	GList *children;
	GtkSpace *space;
	GtkSpaceChild *child;
  //g_print("%s\n", __FUNCTION__);

	widget->allocation = *allocation;
	space = GTK_SPACE(widget);

	children = space->children;
	while(children) {
		child = children->data;
		children = children->next;
		child->widget->allocation.x = child->x;
		child->widget->allocation.y = child->y;
		child->widget->allocation.width = child->widget->requisition.width;
		child->widget->allocation.height = child->widget->requisition.height;
		gtk_widget_size_allocate(child->widget, &child->widget->allocation);
	}
}

static void
gtk_space_size_request (GtkWidget      *widget,
		       GtkRequisition *requisition)
{
	GList *children;
	GtkSpace *space;
	GtkSpaceChild *child;
  //g_print("%s\n", __FUNCTION__);

	space = GTK_SPACE(widget);

	children = space->children;
	while(children) {
		child = children->data;
		children = children->next;
		//child->widget->allocation.x = child->x;
		//child->widget->allocation.y = child->y;
		gtk_widget_size_request(child->widget, &child->widget->requisition);
	}
	//requisition->width = 500 + 0*widget->parent->allocation.width;
	//requisition->height = 500 + 0*widget->parent->allocation.height;
}

static gint
gtk_space_expose (GtkWidget      *widget,
		GdkEventExpose *event)
{
  GtkSpace *space;
  GtkSpaceChild *child;
  GdkEventExpose child_event;
  GList *children;
  //g_print("%s\n", __FUNCTION__);

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_SPACE (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (GTK_WIDGET_DRAWABLE (widget))
    {
      space = GTK_SPACE (widget);

      child_event = *event;

      children = space->children;
      while (children)
	{
	  child = children->data;
	  children = children->next;

	  if (GTK_WIDGET_NO_WINDOW (child->widget) &&
	      gtk_widget_intersect (child->widget, &event->area, &child_event.area))
	    gtk_widget_event (child->widget, (GdkEvent*) &child_event);
	}
    }

  return FALSE;
}

static void
gtk_space_draw (GtkWidget    *widget,
	      GdkRectangle *area)
{
  GtkSpace *space;
  GtkSpaceChild *child;
  GdkRectangle child_area;
  GList *children;
  //g_print("%s\n", __FUNCTION__);

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_SPACE (widget));

  if (GTK_WIDGET_DRAWABLE (widget))
    {
      space = GTK_SPACE (widget);

      children = space->children;
      while (children)
	{
	  child = children->data;
	  children = children->next;

	  if (gtk_widget_intersect (child->widget, area, &child_area))
	    gtk_widget_draw (child->widget, &child_area);
	}
    }
}

static void
gtk_space_map (GtkWidget *widget)
{
  GtkSpace *space;
  GtkSpaceChild *child;
  GList *children;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_SPACE (widget));

  space = GTK_SPACE (widget);
  GTK_WIDGET_SET_FLAGS (space, GTK_MAPPED);

  children = space->children;
  while (children)
    {
      child = children->data;
      children = children->next;

      if (GTK_WIDGET_VISIBLE (child->widget) &&
	  !GTK_WIDGET_MAPPED (child->widget))
	gtk_widget_map (child->widget);
    }
}

static void
gtk_space_unmap (GtkWidget *widget)
{
  GtkSpace *space;
  GtkSpaceChild *child;
  GList *children;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_SPACE (widget));

  space = GTK_SPACE (widget);
  GTK_WIDGET_UNSET_FLAGS (space, GTK_MAPPED);

  children = space->children;
  while (children)
    {
      child = children->data;
      children = children->next;

      if (GTK_WIDGET_VISIBLE (child->widget) &&
	  GTK_WIDGET_MAPPED (child->widget))
	gtk_widget_unmap (child->widget);
    }
}
