/* 

   command center to driver  TiMidity -- Experimental MIDI to WAVE converter
   Copyright (C) 1995 Tuukka Toivonen <toivonen@clinet.fi>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>

#include "timidity_c.h"
#include "BITMAPS/back.xbm"
#include "BITMAPS/next.xbm"
#include "BITMAPS/prev.xbm"
#include "BITMAPS/restart.xbm"
#include "BITMAPS/fwd.xbm"
#include "BITMAPS/pause.xbm"
#include "BITMAPS/quit.xbm"
#if 0
#include "BITMAPS/timidity.xbm"
#endif

#define MIDI_DIR "~/midi/"


void list_append (GtkWidget * file_list, char *filename);
extern char *basename (char *);
void create_message_box (GtkWidget * vbox);
void create_control_button (GtkWidget *);
void create_dialog_boxes (void);
void create_file_list (GtkWidget *);
void create_volume_bar (GtkWidget *);
int fork_player(char *, char **);
GtkWidget *toplevel;

GtkWidget *logwindow = NULL;


GtkWidget *open_dialog = NULL;

extern int fpip_in;

GtkWidget *counterlbl, *totlbl;
GtkWidget *labels[500];
GtkWidget *message_box;
char selecteddir[500];
gint timeout_tag = 0, idle_tag = 0;
int last_sec = 0;
int max_sec = 0;
int maxline = 0, curline = 0;
int scale_freeze = FALSE;
int locator_value;
int not_play = FALSE;

GtkWidget *file_namelbl;

GtkWidget *volume_scale, *locator_scale;
GtkWidget *auto_next_option;

int locator_scale_button = GDK_BUTTON_RELEASE;
int volume_scale_button = GDK_BUTTON_RELEASE;

GtkWidget *file_list = NULL;
int file_number_to_play;	/* Number of the file we're playing in the list */
int nfiles = 0;
char *filenames[300];

/************************************
 *                                  *
 * ALL THE INTERFACE'S CALLBACKS    *
 *                                  *
 ************************************/
void writelog (char *s);

void
destroy_window (GtkWidget * w, GtkWidget ** window)
{
  *window = NULL;
}

gint
clean_message_timer (gpointer foo)
{
  int i;
  for (i = curline; i < maxline; i++)
    {
      gtk_label_set (GTK_LABEL (labels[i]), "");
      gtk_widget_draw (labels[i], NULL);
    }
  gtk_idle_remove (idle_tag);
  idle_tag = 0;
  return TRUE;
}

gint
message_timer (gpointer foo)
{
  gtk_container_enable_resize (GTK_CONTAINER (message_box));
  gtk_timeout_remove (timeout_tag);
  timeout_tag = 0;
  if (idle_tag)
    {
      clean_message_timer (NULL);
    }
  idle_tag = gtk_idle_add ((GtkFunction) clean_message_timer, NULL);
  return TRUE;
}

void
play_init (void)
{
  GtkAdjustment *adjustment;
  /* avoid flickering of message box */
  gtk_container_disable_resize (GTK_CONTAINER (message_box));
  curline = 0;
  if (timeout_tag)
    {
      message_timer (NULL);
    }
  timeout_tag = gtk_timeout_add (500, message_timer, NULL);

  /* restore message box to first page */
  adjustment = gtk_range_get_adjustment (
					  GTK_RANGE (GTK_SCROLLED_WINDOW (
				 message_box->parent->parent)->vscrollbar));
  adjustment->value = 0;
  gtk_signal_emit_by_name (GTK_OBJECT (adjustment), "changed");
}

/*
 * Generic buttons callbacks ( Transport Buttons )
 */
void
GenericCB (GtkWidget * widget, gpointer user_data)
{
  if (not_play)
    {
      not_play = FALSE;
      switch ((int) user_data)
	{
	case MOTIF_RESTART:
	case MOTIF_FWD:
	case MOTIF_RWD:
	  if (file_number_to_play < 0)
	    return;
	  pipe_int_write (MOTIF_PLAY_FILE);
	  pipe_string_write (filenames[file_number_to_play]);
	  play_init ();
	  break;
	default:
	  break;
	}
    }
  pipe_int_write ((int) user_data);
}

/*
 *  Generic scales callbacks : VOLUME and LOCATOR
 */
void
Generic_scaleCB (GtkWidget * widget, gpointer user_data)
{
  if (locator_scale_button == GDK_BUTTON_PRESS)
    return;
  if (volume_scale_button == GDK_BUTTON_PRESS)
    return;
  pipe_int_write ((int) user_data);
  pipe_int_write ((int) (GTK_ADJUSTMENT (widget)->value));
}

/* 
 * Detect when a mouse button is pushed or released in a scale area to
 * avoid concurent scale value modification while holding it with the mouse
 */
void
Locator_btn (GtkWidget * w, GdkEventButton * event)
{
  locator_scale_button = event->type;
  if (locator_scale_button == GDK_BUTTON_PRESS)
    return;
  locator_value = GTK_RANGE (w)->adjustment->value;
  if (not_play)
    {
      not_play = FALSE;
      if (file_number_to_play < 0)
	return;
      pipe_int_write (MOTIF_PLAY_FILE);
      pipe_string_write (filenames[file_number_to_play]);
      play_init ();
      scale_freeze = TRUE;
    }
  pipe_int_write (MOTIF_CHANGE_LOCATOR);
  pipe_int_write (locator_value);
  scale_freeze = TRUE;
}

void
Volume_btn (GtkWidget * w, GdkEventButton * event)
{
  volume_scale_button = event->type;
}

/*
 * File List selection CALLBACK
 */
void
File_ListCB (GtkWidget * widget, GdkEventButton * event, gpointer user_data)
{
  GtkWidget *event_widget = gtk_get_event_widget ((GdkEvent *) event);
  int position = (int) gtk_object_get_user_data (GTK_OBJECT (event_widget));
  char *filename = filenames[position];
  pipe_int_write (MOTIF_PLAY_FILE);
  pipe_string_write (filename);
  play_init ();
  file_number_to_play = position;
}

/*
 * Generic menu callback
 */
void
menuCB (GtkWidget * w, gpointer user_data)
{
  switch ((int) user_data)
    {
    case MENU_OPEN:
      {
	if (open_dialog)
	  gtk_widget_map (open_dialog);
	else
	  create_dialog_boxes ();
      }
      break;

    case MENU_QUIT:
      {
	pipe_int_write (MOTIF_QUIT);
      }
      break;
    case MENU_TOGGLE:
      {
      }
      break;
    }
}

void
getfilename (GtkWidget * w, gpointer user_data)
{
  char fullname[500];
  char *filename = gtk_object_get_user_data (GTK_OBJECT (w));
  if (strstr (filename, ".mid") == NULL)
    return;
  strcpy (fullname, selecteddir);
  strcat (fullname, gtk_object_get_user_data (GTK_OBJECT (w)));
  list_append (file_list, fullname);
}

/* 
 * File selection box callback 
 */
void
openCB (GtkWidget * w, gpointer user_data)
{
  switch ((int) user_data)
    {
    case DIALOG_CANCEL:
      gtk_widget_unmap (open_dialog);
      break;
    case DIALOG_ADD:
      list_append (file_list,
		   gtk_file_selection_get_filename (
					 GTK_FILE_SELECTION (open_dialog)));
      break;
    case DIALOG_ALL:
      {
	char *c;
	gtk_container_disable_resize (GTK_CONTAINER (file_list));
	strcpy (selecteddir,
	gtk_file_selection_get_filename (GTK_FILE_SELECTION (open_dialog)));
	if ((c = strrchr (selecteddir, '/')))
	  *(c + 1) = 0;
	gtk_container_foreach (GTK_CONTAINER (
	   GTK_FILE_SELECTION (open_dialog)->file_list), getfilename, NULL);
	gtk_container_enable_resize (GTK_CONTAINER (file_list));
      }
      break;
    default:
      /* get filename from file selection box and add it to the list */
      list_append (file_list,
		   gtk_file_selection_get_filename (
					 GTK_FILE_SELECTION (open_dialog)));
      gtk_widget_unmap (open_dialog);
      break;
    }
}

void
list_append (GtkWidget * file_list, char *filename)
{
  GtkWidget *list_item
  = gtk_list_item_new_with_label (filename);
  gtk_container_add (GTK_CONTAINER (file_list), list_item);
  filenames[nfiles] = g_strdup (filename);
  gtk_object_set_user_data (GTK_OBJECT (list_item), (gpointer) nfiles);
  nfiles++;
  gtk_widget_show (list_item);
}

/********************************************************
 *                                                      *
 * Receive DATA sent by the application on the pipe     *
 *                                                      *
 ********************************************************/
void
handle_input (gpointer data, int source, GdkInputCondition condition)
{
  int message;

  pipe_int_read (&message);
  printf("input %d\n",message);

  switch (message)
    {
    case REFRESH_MESSAGE:
      {
	printf ("REFRESH MESSAGE IS OBSOLETE !!!\n");
      }
      break;

    case TOTALTIME_MESSAGE:
      {
	int cseconds;
	int minutes, seconds;
	char local_string[20];

	pipe_int_read (&cseconds);

	seconds = cseconds / 100;
	minutes = seconds / 60;
	seconds -= minutes * 60;
	sprintf (local_string, "/%i:%02i", minutes, seconds);
	gtk_label_set (GTK_LABEL (totlbl), local_string);

	/* Readjust the time scale */
	GTK_RANGE (locator_scale)->adjustment->value = 0;
	GTK_RANGE (locator_scale)->adjustment->upper = cseconds;
	GTK_RANGE (locator_scale)->adjustment->step_increment = cseconds / 50;
	gtk_widget_draw (GTK_WIDGET (locator_scale), NULL);
	max_sec = cseconds;
      }
      break;

    case MASTERVOL_MESSAGE:
      {
	int volume;

	pipe_int_read (&volume);
	gtk_range_get_adjustment (GTK_RANGE (volume_scale))->value = volume;
	gtk_range_slider_update (GTK_RANGE (volume_scale));
	gtk_widget_draw (GTK_WIDGET (volume_scale), NULL);
      }
      break;

    case FILENAME_MESSAGE:
      {
	char filename[255];
	char *pc;

	pipe_string_read (filename);

	/* Extract basename of the file */
	pc = basename (filename);
	gtk_label_set (GTK_LABEL (file_namelbl), pc);
	gtk_window_set_title (GTK_WINDOW (toplevel), pc);

      }
      break;

    case FILE_LIST_MESSAGE:
      {
	char filename[255];
	int i, number_of_files;

	/* reset the playing list : play from the start */
	file_number_to_play = -1;

	pipe_int_read (&number_of_files);

	gtk_container_disable_resize (GTK_CONTAINER (file_list));
	for (i = 0; i < number_of_files; i++)
	  {
	    pipe_string_read (filename);
	    list_append (file_list, filename);
	  }
	gtk_container_enable_resize (GTK_CONTAINER (file_list));
      }
      break;

    case NEXT_FILE_MESSAGE:
    case PREV_FILE_MESSAGE:
    case TUNE_END_MESSAGE:
      {

	/* When a file ends, launch next if auto_next toggle */

	if ((message == TUNE_END_MESSAGE) &&
	    (GTK_TOGGLE_BUTTON (auto_next_option)->active == FALSE))
	  {
	    not_play = TRUE;
	    return;
	  }
	if (nfiles == 0)
	  return;

	play_init ();

	/* Total number of file to play in the list */

	if (message == PREV_FILE_MESSAGE)
	  file_number_to_play--;
	else
	  file_number_to_play++;

	/* Do nothing if requested file is before first one */
	if (file_number_to_play < 0)
	  file_number_to_play = nfiles - 1;

	/* Stop after playing the last file */
	if (file_number_to_play >= nfiles)
	  file_number_to_play = 0;

	gtk_list_select_item (GTK_LIST (file_list), file_number_to_play);
	{
	  gfloat position, page_size, upper, one_height;
	  GtkAdjustment *adjustment = gtk_viewport_get_vadjustment (
					  GTK_VIEWPORT (file_list->parent));
	  upper = adjustment->upper;
	  page_size = adjustment->page_size;
	  one_height = upper / nfiles;
	  position = one_height * file_number_to_play;
	  if ((position > adjustment->value) &&
	      (position < (adjustment->value + page_size - one_height)))
	    {
	      /* this item is visible */
	    }
	  else
	    {
	      if (position < page_size / 2)
		{
		  adjustment->value = 0.0;
		}
	      else if (position > upper - page_size / 2)
		{
		  adjustment->value = upper - page_size;
		}
	      else
		{
		  adjustment->value = position - page_size / 2;
		}
	      gtk_signal_emit_by_name (GTK_OBJECT (adjustment), "value_changed");
	    }
	}
	pipe_int_write (MOTIF_PLAY_FILE);
	pipe_string_write (filenames[file_number_to_play]);
      }
      break;

    case CURTIME_MESSAGE:
      {
	int cseconds;
	int sec, seconds, minutes;
	int nbvoice;
	char local_string[20];

	pipe_int_read (&cseconds);
	pipe_int_read (&nbvoice);

	sec = seconds = cseconds / 100;

	/* To avoid blinking */
	if (sec != last_sec)
	  {
	    minutes = seconds / 60;
	    seconds -= minutes * 60;

	    sprintf (local_string, "%2d:%02d",
		     minutes, seconds);
	    gtk_label_set (GTK_LABEL (counterlbl), local_string);
	  }
	last_sec = sec;

	/* Readjust the time scale if not dragging the scale */
	if ((locator_scale_button == GDK_BUTTON_RELEASE)
	    && (cseconds <= max_sec))
	  {
	    gint old;
	    if (scale_freeze)
	      scale_freeze = FALSE;
	    else
	      {
		GtkAdjustment *adjustment =
		GTK_RANGE (locator_scale)->adjustment;
		old = adjustment->value;
		if (old != 50 * (cseconds / 50))
		  {
		    adjustment->value = 50 * (cseconds / 50);
		    gtk_range_slider_update (GTK_RANGE (locator_scale));
		  }
	      }
	  }
      }
      break;

    case NOTE_MESSAGE:
      {
	int channel;
	int note;

	pipe_int_read (&channel);
	pipe_int_read (&note);
	printf ("NOTE chn%i %i\n", channel, note);
      }
      break;

    case PROGRAM_MESSAGE:
      {
	int channel;
	int pgm;

	pipe_int_read (&channel);
	pipe_int_read (&pgm);
	printf ("NOTE chn%i %i\n", channel, pgm);
      }
      break;

    case VOLUME_MESSAGE:
      {
	int channel;
	int volume;

	pipe_int_read (&channel);
	pipe_int_read (&volume);
	printf ("VOLUME= chn%i %i \n", channel, volume);
      }
      break;


    case EXPRESSION_MESSAGE:
      {
	int channel;
	int express;

	pipe_int_read (&channel);
	pipe_int_read (&express);
	printf ("EXPRESSION= chn%i %i \n", channel, express);
      }
      break;

    case PANNING_MESSAGE:
      {
	int channel;
	int pan;

	pipe_int_read (&channel);
	pipe_int_read (&pan);
	printf ("PANNING= chn%i %i \n", channel, pan);
      }
      break;

    case SUSTAIN_MESSAGE:
      {
	int channel;
	int sust;

	pipe_int_read (&channel);
	pipe_int_read (&sust);
	printf ("SUSTAIN= chn%i %i \n", channel, sust);
      }
      break;

    case PITCH_MESSAGE:
      {
	int channel;
	int bend;

	pipe_int_read (&channel);
	pipe_int_read (&bend);
	printf ("PITCH BEND= chn%i %i \n", channel, bend);
      }
      break;

    case RESET_MESSAGE:
      {
	printf ("RESET_MESSAGE\n");
      }
      break;

    case CLOSE_MESSAGE:
      {
	gtk_exit (0);
      }
      break;

    case CMSG_MESSAGE:
      {
	int type;
	char message[1000];

	pipe_int_read (&type);
	pipe_string_read (message);

	writelog (message);
	writelog ("\n");
      }
      break;
    default:
      fprintf (stderr, "UNKNOW MOTIF MESSAGE %i\n", message);
    }

}

/* *******************************************
 *                                           *
 * Interface initialisation before launching *
 *                                           *
 *********************************************/

void
create_menus (GtkWidget * box)
{
  GtkWidget *hbox, *button;

  hbox = gtk_hbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (box), hbox);
  gtk_widget_show (hbox);

  button = gtk_button_new_with_label ("Load Midi Files");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc) menuCB, (gpointer) MENU_OPEN);
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  gtk_widget_show (button);

  auto_next_option = gtk_check_button_new_with_label ("AUTO_NEXT");
  gtk_signal_connect (GTK_OBJECT (auto_next_option), "toggled",
		      (GtkSignalFunc) menuCB, (gpointer) MENU_TOGGLE);
  gtk_box_pack_start (GTK_BOX (hbox), auto_next_option, FALSE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (auto_next_option), TRUE);
  gtk_widget_show (auto_next_option);
}

void
create_dialog_boxes (void)
{
  /* create the file selection box used by MENU_OPEN */
  GList *children;
  GtkWidget *help_button;
  GtkWidget *button;
  open_dialog = gtk_file_selection_new ("Open Midi Files");
  gtk_window_position (GTK_WINDOW (open_dialog), GTK_WIN_POS_MOUSE);
  gtk_signal_connect (GTK_OBJECT (open_dialog), "destroy",
		      (GtkSignalFunc) destroy_window, &open_dialog);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (open_dialog)->ok_button),
		   "clicked", (GtkSignalFunc) openCB, (gpointer) DIALOG_OK);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (open_dialog)->cancel_button),
	       "clicked", (GtkSignalFunc) openCB, (gpointer) DIALOG_CANCEL);
  help_button = GTK_FILE_SELECTION (open_dialog)->help_button;
  children = gtk_container_children (GTK_CONTAINER (help_button));
  gtk_label_set (GTK_LABEL (children->data), "Add");
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (open_dialog)->help_button),
		  "clicked", (GtkSignalFunc) openCB, (gpointer) DIALOG_ADD);
  button = gtk_button_new_with_label ("Add *.mid");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (
		     GTK_FILE_SELECTION (open_dialog)->help_button->parent),
		      button, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc) openCB, (gpointer) DIALOG_ALL);
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (open_dialog), MIDI_DIR);
  gtk_widget_show (button);
  gtk_widget_show (open_dialog);
}

void
main(int argc, char *argv[])
{
  GtkObject *adjustment;
  GtkWidget *vbox, *hbox;
  GtkWidget *frame;
  GtkWidget *separator;
  GtkWidget *infobox, *timeinfo;
  GtkWidget *timebox;
  char *playerargv[] = {
	"timidity",
	"-il",
	"a",
	NULL,
  };
  char *player = playerargv[0];

  /* create the toplevel shell */

  if(!fork_player(player, playerargv)) {
	exit(1);
  };

  gtk_init (&argc, NULL);
  toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (toplevel), "Timidity");

  vbox = gtk_vbox_new (FALSE, 10);
  gtk_container_add (GTK_CONTAINER (toplevel), vbox);
  gtk_container_border_width (GTK_CONTAINER (vbox), 5);
  gtk_widget_show (vbox);

  create_menus (vbox);

  /* Control buttons in a form */
/*****************************/

  /* create form for the row of control buttons */

  create_control_button (vbox);


  separator = gtk_hseparator_new ();
  gtk_container_add (GTK_CONTAINER (vbox), separator);
  gtk_widget_show (separator);

  hbox = gtk_hbox_new (FALSE, 10);
  gtk_container_border_width (GTK_CONTAINER (hbox), 10);
  gtk_container_add (GTK_CONTAINER (vbox), hbox);
  gtk_widget_show (hbox);
/********************/
  /* File_name label  */
/********************/

  infobox = gtk_vbox_new (FALSE, 10);
  gtk_container_add (GTK_CONTAINER (hbox), infobox);
  gtk_widget_show (infobox);

  frame = gtk_frame_new (" Playing   ");
  gtk_container_add (GTK_CONTAINER (infobox), frame);
  gtk_container_border_width (GTK_CONTAINER (frame), 5);
  gtk_widget_show (frame);
  {
    GtkWidget *box = gtk_vbox_new (FALSE, 5);
    gtk_container_add (GTK_CONTAINER (frame), box);
    gtk_widget_show (box);
    file_namelbl = gtk_label_new ("                     ");
    gtk_container_add (GTK_CONTAINER (box), file_namelbl);
    gtk_widget_show (file_namelbl);
  }


/*****************************/
  /* TIME LABELS IN A FORM     */
/*****************************/



  /* Total time label */
  frame = gtk_frame_new ("Time");
  gtk_container_add (GTK_CONTAINER (infobox), frame);
  gtk_container_border_width (GTK_CONTAINER (frame), 5);
  gtk_widget_show (frame);

  timebox = gtk_vbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (frame), timebox);
  gtk_widget_show (timebox);

  timeinfo = gtk_hbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (timebox), timeinfo);
  gtk_widget_show (timeinfo);


  /* current time label */
  counterlbl = gtk_label_new ("00:00");
  gtk_misc_set_alignment (GTK_MISC (counterlbl), 1.0, 0.5);
  gtk_container_add (GTK_CONTAINER (timeinfo), counterlbl);
  /*
     gtk_box_pack_end(GTK_BOX(timeinfo), counterlbl, FALSE, FALSE, 0);
   */
  gtk_widget_show (counterlbl);

  totlbl = gtk_label_new ("/00:00");
  gtk_misc_set_alignment (GTK_MISC (totlbl), 0.0, 0.5);
  gtk_container_add (GTK_CONTAINER (timeinfo), totlbl);
  gtk_widget_show (totlbl);

/******************/
  /* Locator Scale  */
/******************/
  adjustment = gtk_adjustment_new (0, 0, 110, 1, 10, 10);
  gtk_signal_connect (GTK_OBJECT (adjustment), "value_changed",
	  (GtkSignalFunc) Generic_scaleCB, (gpointer) MOTIF_CHANGE_LOCATOR);

  locator_scale = gtk_hscale_new (GTK_ADJUSTMENT (adjustment));
  gtk_signal_connect (GTK_OBJECT (locator_scale), "button_release_event",
		      (GtkSignalFunc) Locator_btn, NULL);
  gtk_signal_connect (GTK_OBJECT (locator_scale), "button_press_event",
		      (GtkSignalFunc) Locator_btn, NULL);
  gtk_scale_set_draw_value (GTK_SCALE (locator_scale), FALSE);
  gtk_container_add (GTK_CONTAINER (timebox), locator_scale);
  gtk_widget_show (locator_scale);


  create_volume_bar (infobox);
  create_file_list (hbox);

  separator = gtk_hseparator_new ();
  gtk_container_add (GTK_CONTAINER (vbox), separator);
  gtk_widget_show (separator);

/*******************/
  /* Message window  */
/*******************/

  create_message_box (vbox);

/*******************************************************/
  /* Plug the pipe ..... and heeere we go                */
/*******************************************************/

  gdk_input_add (fpip_in, GDK_INPUT_READ, handle_input, NULL);

  gtk_widget_show (toplevel);
  gtk_main ();
}

void
pixmapbutton (GtkWidget * box, char *data, int width, int height,
	      GtkSignalFunc callback, gpointer cont)
{
  GtkWidget *button = gtk_button_new ();
  GdkPixmap *pixmap1, *pixmap2, *pixmap3;
  GtkWidget *pixmapwid;
  GdkColor *fg, *bg;
  GtkStyle *style;
  GdkColormap *colormap =
  gdk_window_get_colormap ((GdkWindow *) & gdk_root_parent);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      callback, cont);
  gtk_container_add (GTK_CONTAINER (box), button);
  gtk_widget_show (button);

  style = gtk_widget_get_style (button);

  fg = &style->fg[GTK_STATE_NORMAL];
  bg = &style->bg[GTK_STATE_NORMAL];
  gdk_color_alloc (colormap, fg);
  gdk_color_alloc (colormap, bg);
  pixmap1 = gdk_pixmap_create_from_data (NULL, data, width, height,
					 -1, fg, bg);

  fg = &style->fg[GTK_STATE_ACTIVE];
  bg = &style->bg[GTK_STATE_ACTIVE];
  gdk_color_alloc (colormap, fg);
  gdk_color_alloc (colormap, bg);
  pixmap2 = gdk_pixmap_create_from_data (NULL, data, width, height,
					 -1, fg, bg);

  fg = &style->fg[GTK_STATE_PRELIGHT];
  bg = &style->bg[GTK_STATE_PRELIGHT];
  gdk_color_alloc (colormap, fg);
  gdk_color_alloc (colormap, bg);
  pixmap3 = gdk_pixmap_create_from_data (NULL, data, width, height,
					 -1, fg, bg);
  pixmapwid = gtk_pixmap_new (pixmap1, pixmap2, pixmap3, NULL, NULL);
  gtk_container_add (GTK_CONTAINER (button), pixmapwid);
  gtk_widget_show (pixmapwid);
}

void
create_control_button (GtkWidget * box)
{
  GtkWidget *hbox = gtk_hbox_new (TRUE, 10);
  gtk_container_add (GTK_CONTAINER (box), hbox);
  gtk_container_border_width (GTK_CONTAINER (hbox), 10);
  gtk_widget_show (hbox);

  /* Previous Button */
  pixmapbutton (hbox, prev_bits, prev_width, prev_height,
		(GtkSignalFunc) GenericCB, (gpointer) MOTIF_PREV);

  /* Backward Button */
  pixmapbutton (hbox, back_bits, back_width, back_height,
		(GtkSignalFunc) GenericCB, (gpointer) MOTIF_RWD);

  /* Restart Button */
  pixmapbutton (hbox, restart_bits, restart_width, restart_height,
		(GtkSignalFunc) GenericCB, (gpointer) MOTIF_RESTART);

  /* Quit Button */
  pixmapbutton (hbox, quit_bits, quit_width, quit_height,
		(GtkSignalFunc) GenericCB, (gpointer) MOTIF_QUIT);

  /* Pause Button */
  pixmapbutton (hbox, pause_bits, pause_width, pause_height,
		(GtkSignalFunc) GenericCB, (gpointer) MOTIF_PAUSE);

  /* Forward Button */
  pixmapbutton (hbox, fwd_bits, fwd_width, fwd_height,
		(GtkSignalFunc) GenericCB, (gpointer) MOTIF_FWD);

  /* Next Button */
  pixmapbutton (hbox, next_bits, next_width, next_height,
		(GtkSignalFunc) GenericCB, (gpointer) MOTIF_NEXT);
}

void
create_volume_bar (GtkWidget * box)
{
/********************/
  /* Volume scale     */
/********************/
  GtkObject *adjustment;
  GtkWidget *frame;
  frame = gtk_frame_new ("Volume");
  gtk_container_add (GTK_CONTAINER (box), frame);
  gtk_container_border_width (GTK_CONTAINER (frame), 5);
  gtk_widget_show (frame);

  /*
     vbox = gtk_vbox_new(FALSE, 2);
     gtk_container_add(GTK_CONTAINER(frame), vbox);
     gtk_widget_show(vbox);
   */

  adjustment = gtk_adjustment_new (0, 0, 800 + 10, 10, 10, 10);
  gtk_signal_connect (GTK_OBJECT (adjustment), "value_changed",
	   (GtkSignalFunc) Generic_scaleCB, (gpointer) MOTIF_CHANGE_VOLUME);
  volume_scale = gtk_hscale_new (GTK_ADJUSTMENT (adjustment));
  gtk_scale_set_digits (GTK_SCALE (volume_scale), 0);
  gtk_scale_set_value_pos (GTK_SCALE (volume_scale), GTK_POS_TOP);
  gtk_range_set_update_policy (GTK_RANGE (volume_scale), GTK_UPDATE_DELAYED);
  gtk_scale_set_draw_value (GTK_SCALE (volume_scale), FALSE);
  gtk_container_add (GTK_CONTAINER (frame), volume_scale);
  gtk_widget_show (volume_scale);
}

void
create_file_list (GtkWidget * box)
{
  GtkWidget *scrolled_window;
  if (file_list)
    {
      gtk_widget_show (file_list);
      return;
    }
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER (box), scrolled_window);
  gtk_widget_show (scrolled_window);

  file_list = gtk_list_new ();
  gtk_list_set_selection_mode (GTK_LIST (file_list), GTK_SELECTION_BROWSE);
  gtk_container_add (GTK_CONTAINER (scrolled_window), file_list);
  gtk_signal_connect (GTK_OBJECT (file_list), "button_press_event",
		      (GtkSignalFunc) File_ListCB, NULL);
  gtk_widget_show (file_list);
}

#include <signal.h>
void
create_message_box (GtkWidget * box)
{
  GtkWidget *scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_container_add (GTK_CONTAINER (box), scrolled_window);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_widget_set_usize (scrolled_window, 0, 200);
  gtk_widget_show (scrolled_window);
  message_box = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (scrolled_window), message_box);
  gtk_widget_show (message_box);
  writelog ("\n\n\n\n\n\n\n\n\n\n\n\n\n");
}

void
writelog (char *ss)
{
  char *s = g_strdup (ss);
  char *start, *realstart;
  static char save[200];
  if (curline == 1)
    {
      if ((file_number_to_play >= 0)
	  && (strncmp (s, filenames[file_number_to_play],
		       strlen (filenames[file_number_to_play])) == 0))
	{
	  /* error: invalid midi file */
	  int i;
	  g_free (filenames[file_number_to_play]);
	  for(i=file_number_to_play;i<nfiles -1;i++) {
		filenames[i] = filenames[i+1];
	  }
	  /*
	  gtk_container_remove(GTK_CONTAINER(file_list), g_list_nth(GTK_LIST(file_list)->children, file_number_to_play)->data);
	  */
	  gtk_list_clear_items (GTK_LIST (file_list), file_number_to_play,
				file_number_to_play + 1);
	  file_number_to_play--;
	  nfiles--;
	}
    }
  start = realstart = s;

  while ((s = strchr (s, '\n')))
    {
      *s = '\0';
      strcat (save, start);
      if (maxline == curline)
	{
	  labels[maxline] = gtk_label_new (save);
	  gtk_misc_set_alignment (GTK_MISC (labels[maxline]), 0.0, 0.5);
	  gtk_box_pack_start (GTK_BOX (message_box), labels[maxline],
			      FALSE, FALSE, 0);
	  gtk_widget_show (labels[maxline]);
	  maxline++;
	}
      else
	{
	  gtk_label_set (GTK_LABEL (labels[curline]), save);
	  gtk_widget_draw (labels[curline], NULL);
	}
      curline++;
      save[0] = 0;
      start = ++s;
    }
  strcat (save, start);
  g_free (realstart);
}
