
/* this file is a part of sori software, (C) Hwang chi-duck 1997               */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include "sori.h"
#include "timidity_c.h"



int last_sec;
int max_sec;


void 
timidity_button_callback(GtkWidget * widget, gpointer user_data)
{
	timidity_int_write((int) user_data);
}

void 
timidity_change_position(float percent)
{
	timidity_int_write(MOTIF_CHANGE_LOCATOR);
	timidity_int_write((int)(percent*max_sec));
}

void 
timidity_change_volume(float p)
{
	timidity_int_write(MOTIF_CHANGE_VOLUME);
	timidity_int_write((int)(8*p));
}

void
timidity_pause(void)
{
	timidity_int_write(MOTIF_PAUSE);
}

void 
timidity_play_song(char *filename)
{
	int fd = open(filename, O_RDONLY);
	if(fd == -1) {
		gdk_beep();
		g_warning("can't open %s", filename);
		return;
	}
	close(fd);
	timidity_int_write(MOTIF_PLAY_FILE);
	timidity_string_write(filename);
}

/********************************************************
 *                                                      *
 * Receive DATA sent by the timidity on the pipe     *
 *                                                      *
 ********************************************************/
void timidity_handle_input(void)
{
	int message;

	timidity_int_read(&message);

	switch (message) {
	case REFRESH_MESSAGE:
		{
			printf("REFRESH MESSAGE IS OBSOLETE !!!\n");
		}
		break;

	case TOTALTIME_MESSAGE:
		{
			int second;

			timidity_int_read(&second);

			second = second / 100;
			//show_total_play_time(second);
			max_sec = second;
		}
		break;

	case MASTERVOL_MESSAGE:
		{
			int volume;

			timidity_int_read(&volume);
#if 0
			show_volume(volume/8);
			gtk_range_get_adjustment(GTK_RANGE(volume_scale))->value = volume;
			gtk_range_slider_update(GTK_RANGE(volume_scale));
			gtk_widget_draw(GTK_WIDGET(volume_scale), NULL);
#endif
		}
		break;

	case FILENAME_MESSAGE:
		{
			char filename[255];

			timidity_string_read(filename);


		}
		break;

	case FILE_LIST_MESSAGE:
		{
			char filename[255];
			int i, number_of_files;

			/* reset the playing list : play from the start */

			timidity_int_read(&number_of_files);
			for (i = 0; i < number_of_files; i++) {
				timidity_string_read(filename);
			}

#if 0
			extern int current_file;
			current_file = -1;
			/* we have list of song already */
			/* so, discard those */
			list_append_begin();
			for (i = 0; i < number_of_files; i++) {
				timidity_string_read(filename);
				list_append(filename);
			}
			list_append_end();
#endif
		}
		break;

	case NEXT_FILE_MESSAGE:
#if 0
		play_next();
#endif
		break;
	case PREV_FILE_MESSAGE:
#if 0
		play_previous();
#endif
		break;
	case TUNE_END_MESSAGE:
		play_end();
		break;
	case CURTIME_MESSAGE:
		{
			int second;
			int nbvoice;

			timidity_int_read(&second);
			timidity_int_read(&nbvoice);

			//g_print("%d %d\n", second, max_sec);
			show_position(100.0*((float)second)/max_sec);
		}
		break;

	case NOTE_MESSAGE:
		{
			int channel;
			int note;

			timidity_int_read(&channel);
			timidity_int_read(&note);
#if 0
			printf("NOTE chn%i %i\n", channel, note);
#endif
		}
		break;

	case PROGRAM_MESSAGE:
		{
			int channel;
			int pgm;

			timidity_int_read(&channel);
			timidity_int_read(&pgm);
#if 0
			printf("NOTE chn%i %i\n", channel, pgm);
#endif
		}
		break;

	case VOLUME_MESSAGE:
		{
			int channel;
			int volume;

			timidity_int_read(&channel);
			timidity_int_read(&volume);
#if 0
			printf("VOLUME= chn%i %i \n", channel, volume);
#endif
		}
		break;


	case EXPRESSION_MESSAGE:
		{
			int channel;
			int express;

			timidity_int_read(&channel);
			timidity_int_read(&express);
#if 0
			printf("EXPRESSION= chn%i %i \n", channel, express);
#endif
		}
		break;

	case PANNING_MESSAGE:
		{
			int channel;
			int pan;

			timidity_int_read(&channel);
			timidity_int_read(&pan);
#if 0
			printf("PANNING= chn%i %i \n", channel, pan);
#endif
		}
		break;

	case SUSTAIN_MESSAGE:
		{
			int channel;
			int sust;

			timidity_int_read(&channel);
			timidity_int_read(&sust);
#if 0
			printf("SUSTAIN= chn%i %i \n", channel, sust);
#endif
		}
		break;

	case PITCH_MESSAGE:
		{
			int channel;
			int bend;

			timidity_int_read(&channel);
			timidity_int_read(&bend);
#if 0
			printf("PITCH BEND= chn%i %i \n", channel, bend);
#endif
		}
		break;

	case RESET_MESSAGE:
		{
#if 0
			printf("RESET_MESSAGE\n");
#endif
		}
		break;

	case CLOSE_MESSAGE:
		{
#if 0
			gtk_exit(0);
#endif
		}
		break;

	case CMSG_MESSAGE:
		{
			int type;
			char message[1000];

			timidity_int_read(&type);
			timidity_string_read(message);

			//writelog(message);
			//writelog("\n");
		}
		break;
	default:
		fprintf(stderr, "UNKNOWN MESSAGE FROM TIMIDITY: %i\n", message);
	}

}

void timidity_quit(void)
{
	timidity_int_write(MOTIF_QUIT);
}

void
timidity_forward(void)
{
}

void 
timidity_backward(void)
{

}
