
/* this file is a part of sori software, (C) Hwang chi-duck 1997               */

/* 

   TiMidity -- Experimental MIDI to WAVE converter
   Copyright (C) 1995 Tuukka Toivonen <toivonen@clinet.fi>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   motif_pipe.c: written by Vincent Pagel (pagel@loria.fr) 10/4/95

   pipe communication between motif interface and sound generator

 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <stdarg.h>
#include <sys/ioctl.h>
#ifdef SOLARIS
#include <sys/filio.h>
#endif

extern int fpip_in, fpip_out;			/* in and out depends in which process we are */
int pid;						/* Pid for child process */
int pipe_error = 0;

/* DATA VALIDITY CHECK */
#define INT_CODE 214
#define STRING_CODE 216


/***********************************************************************/
/* PIPE COMUNICATION                                                   */
/***********************************************************************/

void timidity_error(char *st)
{
	extern void player_is_dead(void);
	fprintf(stderr, "CONNECTION PROBLEM WITH MOTIF PROCESS IN %s BECAUSE:%s\n",
			st,
			sys_errlist[errno]);
	pipe_error = 1;
}


/*****************************************
 *              INT                      *
 *****************************************/

void timidity_int_write(int c)
{
	int len;

#ifdef DEBUGPIPE
	int code = INT_CODE;
	len = write(fpip_out, &code, sizeof(code));
	if (len != sizeof(code))
		timidity_error("PIPE_INT_WRITE");
#endif

	len = write(fpip_out, &c, sizeof(c));
	if (len != sizeof(int))
		 timidity_error("PIPE_INT_WRITE");
}

void timidity_int_read(int *c)
{
	int len;

#ifdef DEBUGPIPE
	int code;

	len = read(fpip_in, &code, sizeof(code));
	if (len != sizeof(code))
		timidity_error("PIPE_INT_READ");
	if (code != INT_CODE)
		fprintf(stderr, "BUG ALERT ON INT PIPE %i\n", code);
#endif

	len = read(fpip_in, c, sizeof(int));
	if (len != sizeof(int))
		 timidity_error("PIPE_INT_READ");
}



/*****************************************
 *              STRINGS                  *
 *****************************************/

void timidity_string_write(char *str)
{
	int len, slen;

#ifdef DEBUGPIPE
	int code = STRING_CODE;

	len = write(fpip_out, &code, sizeof(code));
	if (len != sizeof(code))
		timidity_error("PIPE_STRING_WRITE");
#endif

	slen = strlen(str);
	len = write(fpip_out, &slen, sizeof(slen));
	if (len != sizeof(slen))
		timidity_error("PIPE_STRING_WRITE");

	len = write(fpip_out, str, slen);
	if (len != slen)
		timidity_error("PIPE_STRING_WRITE on string part");
}

void timidity_string_read(char *str)
{
	int len, slen;

#ifdef DEBUGPIPE
	int code;

	len = read(fpip_in, &code, sizeof(code));
	if (len != sizeof(code))
		timidity_error("PIPE_STRING_READ");
	if (code != STRING_CODE)
		fprintf(stderr, "BUG ALERT ON STRING PIPE %i\n", code);
#endif

	len = read(fpip_in, &slen, sizeof(slen));
	if (len != sizeof(slen))
		timidity_error("PIPE_STRING_READ");

	len = read(fpip_in, str, slen);
	if (len != slen)
		timidity_error("PIPE_STRING_READ on string part");
	str[slen] = '\0';			/* Append a terminal 0 */
}

int timidity_read_ready()
{
	int num;

	ioctl(fpip_in, FIONREAD, &num);		/* see how many chars in buffer. */
	return num;
}

